package com.sp.blinddate.base;

import com.google.common.collect.ImmutableMap;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 菜单结构 <br>
 * 菜单上的链接地址是每个模块名(PModule.pname)+权限节点数组(PModule.nodes)的第一个元素。<br>
 *
 * @author oloba
 * @see PModule
 *
 */
public enum PMenu {

	///// 以下代码不可改动！！！！！！
	GENERALIZE("概括",PModule.GENERALIZE),
	USER("认证",PModule.AUTHENTICATE),
	VERIFY("审核",PModule.VERIFY),
	SYS_BASE("系统基本配置", PModule.PERMISSION, PModule.ADMIN_USER),
	;
	// 空：代表是第一层。不空：值是上层菜单。
	public final PMenu parent_menu;
	public final String desc;
	public final String icon;
	//模块列表
	public final PModule[] modules;

	PMenu(String desc, PModule... modules) {
		this( desc, "", modules);
	}
	PMenu(String desc, String icon, PModule... modules) {
		this(null, desc, icon, modules);
	}
	PMenu(PMenu parent_menu, String desc, String icon, PModule... modules) {
		this.parent_menu = parent_menu;
		this.desc = desc;
		this.icon = StringUtils.isBlank(icon) ? "restapi":"";
		this.modules = modules;
	}


	/**
	 * 通过模块找到定义的PMenu
	 * @param pModule
	 * @return
	 */
	public static final PMenu get(PModule pModule) {
		PMenu pMenu = null;
		for (PMenu menu : PMenu.values()) {
			for (PModule module : menu.modules) {
				if (module == pModule) {
					pMenu = menu;
					break;
				}
			}
			if (null != pMenu) {
				break;
			}
		}
		return pMenu;
	}
	/**
	 * 根据提供的module_pname获取所属于的PMenu对象。
	 * @param module_pname
	 * @return PMenu OR NULL
	 */
	public static final PMenu parentPMenu(String module_pname) {
		for (PMenu menu : PMenu.values()) {
			for (PModule module : menu.modules) {
				if (StringUtils.equals(module_pname, module.pname)) {
					return menu;
				}
			}
		}
		return null;
	}

	public static List<Map<String, Object>> toListMap() {
		List<Map<String, Object>> list = new ArrayList<>(PMenu.values().length);
		for (PMenu menu : PMenu.values()) {
			list.add(ImmutableMap.<String, Object>builder()
					.put("parent_menu", "")
					.put("desc", menu.desc)
					.put("icon", menu.icon)
					.put("modules", PModule.toListMap(menu.modules))
					.build());
		}
		return list;
	}
}
