package com.sp.blinddate.base;

import com.google.common.collect.ImmutableMap;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 菜单项，一个Enum常量就是一个菜单，可以设置不显示
 * ！！此对象代码不能改动！！
 * 主菜单的菜单配置。
 * 放在代码上，防止用户更改。方便多环境时一次配置到处运行。
 * @author jahe.lai
 *
 */
public enum PModule {
	BGSOUND("bgsound", "背景音乐",
			MNode.of(PNode.MANAGER, "背景音乐列表"),
			MNode.of(PNode.ADD, "添加背景音乐"),
			MNode.of(PNode.STATUS, "上下架")
	),
	WORK_TEMPLATE("work_template", "参考方案",
			MNode.of(PNode.MANAGER, "方案表格列表"),
			MNode.of(PNode.ADD, "新增方案"),
			MNode.of(PNode.EDIT, "编辑方案"),
			MNode.of(PNode.STATUS, "上下架")
	),
	DICTIONARY("dictionary", "标签管理",
			MNode.of(PNode.MANAGER, "字典表格列表"),
			MNode.of(PNode.NEW, "新增标签类型"),
			MNode.of(PNode.ADD, "新增标签"),
			MNode.of(PNode.DEL, "删除标签")
	),
	WEBSOCKET_SRV("ws_srv", "WS交互消息", "websocket_srv",
			MNode.of(PNode.MANAGER, "WS操作首页"),
			MNode.of(PNode.GET, "连接了WS服务的用户列表"),
			MNode.of(PNode.SETTING, "设置websocket消息的自动连接，广播内容")
	),
	WORKS("works", "作品管理",
			MNode.of(PNode.MANAGER, "作品表格列表"),
			MNode.of(PNode.VIEW, "查看作品的图片清单"),
			MNode.of(PNode.VERIFY, "审核"),
			MNode.of(PNode.SET, "推荐给APP首页")
	),
	GRAPHIC_DYNAMIC("graphic_dynamic", "图文动态",
			MNode.of(PNode.MANAGER, "作品表格列表"),
			MNode.of(PNode.VERIFY, "审核图文动态")
	),
	PERMISSION("permission", "权限数据", "permission/base",
			MNode.of(PNode.MANAGER, "权限基础数据")),

	USER_HEAD_IMG_VERIFY("user_head_img_verify","用户头像审核",
			MNode.of(PNode.LIST, "用户头像待审核列表"),
			MNode.of(PNode.VERIFY, "用户头像审核")),

	WITHDRAW("withdraw","用户提现",
			MNode.of(PNode.MANAGER, "提现管理")),

	AUTHENTICATION("AUTHENTICATION","用户认证",
			MNode.of(PNode.LIST, "用户认证列表"),
			MNode.of(PNode.MANAGER, "用户认证管理")),

	USER_MANAGER("user_manager","用户管理",
			MNode.of(PNode.LIST, "用户列表"),
			MNode.of(PNode.MANAGER, "用户管理")),

	GENERALIZE("generalize","系统概括",
			MNode.of(PNode.MANAGER, "用户管理"),
			MNode.of(PNode.QUERY, "统计信息查询")),

	ADMIN_USER("admin_user", "运营用户和权限",
			MNode.of(PNode.MANAGER, "用户，角色列表"),
			MNode.of(PNode.ADD, "新增运营用户，角色"),
			MNode.of(PNode.EDIT, "修改用户数据"),
			MNode.of(PNode.UPDATE, "更改密码"),
			MNode.of(PNode.LOAD, "加载用户，角色权限数据"),
			MNode.of(PNode.SETTING, "配置用户，角色的权限"),
			MNode.of(PNode.DELETE, "移除用户")),

	AUTHENTICATE("user_auth","VIP认证",
				MNode.of(PNode.MANAGER,"付费会员")),

	VERIFY("verify","相亲资料",
				MNode.of(PNode.MANAGER,"审核列表"),
				MNode.of(PNode.DETAIL,"查看"),
				MNode.of(PNode.VERIFY,"审核"),
				MNode.of(PNode.RECOMMAND,"推荐"),
				MNode.of(PNode.WITHDRAWN,"下架"))
	;
	public final String pname; // 模块标识，用于URI的起始位置. AUri.YQA/pname/
	public final String desc; // 描述，菜单名
	public final String icon;	// 菜单左侧显示的图标。 https://element.eleme.cn/#/zh-CN/component/icon
	public final String vue;	// Vue的视图，因为要做动态路由。默认（/pname/index.vue）
	public final boolean display;  // 作为权限控制模块, 只是在访问菜单上不显示出来.
	public final MNode[] nodes;	// 菜单拥有的功能点,第一个功能点将作为链接使用。 /this.pname/pnode.pname


	// 不显示的功能模块
	PModule(String pname, String desc, MNode... nodes) {
		this(pname, desc, "", "", true, nodes);
	}
	PModule(String pname, String desc, boolean display, MNode... nodes) {
		this(pname, desc, "", "", display, nodes);
	}
	PModule(String pname, String desc, String vue, MNode... nodes) {
		this(pname, desc, "", vue, true, nodes);
	}
	PModule(String pname, String desc, String icon, String vue, MNode... nodes) {
		this(pname, desc, icon, vue, true, nodes);
	}

	PModule(String pname, String desc, String icon, String vue,  boolean display, MNode... nodes) {
		this.pname = pname;
		this.desc = desc;
		this.icon = StringUtils.isBlank(icon) ? "icon" : icon;
		this.vue = StringUtils.isBlank(vue) ? pname : vue;
		this.nodes = nodes;
		this.display = display;
	}

	public String getPname() {
		return pname;
	}
	public String getDesc() {
		return desc;
	}
	public String getVue() {
		return vue;
	}
	public String getIcon() {
		return icon;
	}

	public MNode[] getNodes() {
		return nodes;
	}


	public static final PModule get(String pname) {
		PModule[] ms = PModule.values();
		for (PModule m : ms) {
			if (m.pname.equals(pname)) {
				return m;
			}
		}
		return null;
	}

	public static List<Map<String, Object>> toListMap(PModule... modules) {
		List<Map<String, Object>> list = new ArrayList<>(modules.length);
		for (PModule module : modules) {
			list.add(ImmutableMap.<String, Object>builder()
					.put("pname", module.pname)
					.put("desc", module.desc)
					.put("icon", module.icon)
					.put("vue", module.vue)
					.put("display", module.display)
					.put("nodes", module.nodes)
					.build());
		}
		return list;
	}

}
