package com.sp.blinddate.module.childinfo.controller;

import com.github.pagehelper.PageInfo;
import com.sp.blinddate.base.AUri;
import com.sp.blinddate.common.BaseReviewIenum;
import com.sp.blinddate.module.chlidinfo.param.ChildInfoParam;
import com.sp.blinddate.module.chlidinfo.service.ChildInfoService;
import com.sp.blinddate.module.chlidinfo.vo.ChildInfoVO;
import com.sp.blinddate.module.chlidinfo.vo.UserDetailInfoVO;
import com.sp.http.MJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;

/**
 * @Descrption
 * @Author: 十一
 * @Date: 2019/11/25 17:27
 **/
@RestController
@Api(tags = "子女信息审核域")
public class ChildInfoVerifyController {

    @Autowired
    private ChildInfoService childInfoService;

    @ApiOperation(value = "获取相亲列表",notes = "获取相亲列表")
    @PostMapping(AUri.USERCHILDINF_MANAGER)
    public MJson<PageInfo<ChildInfoVO>> getVerifyList(ChildInfoParam param){
        PageInfo<ChildInfoVO> info = childInfoService.getVerifyList(param);
        return MJson.ok(info);
    }
    
    @ApiOperation(value = "查看相亲资料", notes = "查看相亲资料")
    @GetMapping(AUri.USERCHILDINF_DETAIL)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "infoId", value = "用户详情id", paramType = "query", required = true, dataType = "long")
    })
    public MJson<UserDetailInfoVO> getDetailInfo(@Validated @NotNull(message = "用户详情id为空") Long infoId){
        UserDetailInfoVO vo = childInfoService.getDetailInfo(infoId);
        return MJson.ok(vo);
    }


    @ApiOperation(value = "审核操作", notes = "审核操作")
    @GetMapping(AUri.USERCHILDINF_VERIFY)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userInfoId", value = "用户信息id", paramType = "query", required = true, dataType = "long"),
            @ApiImplicitParam(name = "status", value = "状态：2:审核通过，3：审核拒绝，4：下架", paramType = "query", required = true, dataType = "int")
    })
    public MJson<UserDetailInfoVO> doVerify(@Validated @NotNull(message = "用户信息id为空") Long userInfoId,
                                            @Validated @NotNull(message = "审核状态为空") BaseReviewIenum status){
        childInfoService.verifyUserInfo(userInfoId,status);
        return MJson.ok();
    }


}
