package com.sp.blinddate.module.index;

import com.sp.blinddate.base.AUri;
import com.sp.blinddate.base.AdminCache;
import com.sp.blinddate.base.AdminConst;
import com.sp.blinddate.base.LoginType;
import com.sp.blinddate.module.permission.PrivilegeRouterHelper;
import com.sp.blinddate.module.permission.service.IPermissionService;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.module.permission.vo.RouterMenuVO;
import com.sp.blinddate.module.user.entity.AdminUserPO;
import com.sp.blinddate.module.user.service.IAdminUserService;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.util.AdminUtil;
import com.sp.blinddate.util.CaptchaUtil;
import com.sp.blinddate.websocket.Ws;
import com.sp.friendship.module.dynamic.service.IDailyDynamicsService;
import com.sp.friendship.module.order.service.IOrderService;
import com.sp.friendship.module.user.service.IUserService;
import com.sp.friendship.module.works.service.IWorksService;
import com.sp.http.MJson;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.imageio.ImageIO;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 首页控制器，
 * 非权限控制的请求可在此定义
 *
 * @author: jahe.lai
 * @create: 2019-09-26 10:02
 */
@Slf4j
@RestController
@Api(tags = "概括-用户统计信息")
public class IndexController {
    private static final Logger LOG = LoggerFactory.getLogger(IndexController.class);

    @Autowired
    private IAdminUserService adminUserService;
    @Autowired
    IPermissionService permissionService;
    @Autowired
    AdminCache adminCache;

    /**
     * 生成登录验证码
     * @param response
     * @param t 时间串
     * @throws IOException
     */
    @ApiIgnore
    @GetMapping(AUri.CAPTCHA)
    public void captcha(HttpServletResponse response, String t) throws IOException {
        LOG.info("生成验证码t：{}", t);
        response.setHeader("Cache-Control", "no-store, no-cache");
        response.setContentType("image/jpeg");
        // 生成图片验证码
        BufferedImage image = CaptchaUtil.createImage();
        // 生成文字验证码
        String randomText = CaptchaUtil.drawRandomText(image);
        // 保存验证码 有效期两分钟
        AdminCache.imgCode(AdminConst.LOGIN_IMGCODE_PREFIX + t, randomText.toLowerCase());
        ServletOutputStream out = response.getOutputStream();
        ImageIO.write(image, "jpeg", out);
    }

    /**
     * 请求登录
     * @param username
     * @param password
     * @param token
     * @return
     */
    @ApiIgnore
    @PostMapping(AUri.LOGIN)
    public MJson<String> login(String username, String password, String token) {
        log.warn("psw={}", AdminUtil.encode("123456"));
        if (StringUtils.isNotEmpty(token)) {
            return MJson.ok(token);
        }
        return MJson.ok(adminUserService.checkLogin(username, password));
    }

    /**
     * 请求登出
     * @return
     */
    @ApiIgnore
    @GetMapping(AUri.LOGOUT)
    public MJson<String> logout() {
        String account = AdminUtil.getUser().getUsername();
        adminCache.resetPermission(account);
        log.warn("invoke logout {}",account);
        return MJson.ok("成功登出");
    }

    @ApiIgnore
    @GetMapping(AUri.USER_BASE_INFO)
    public MJson<Object> getInfo() {
        AdminUserPO userPO = adminUserService.findInfo(AdminUtil.getUser().getUserId());
        Map<String, Object> map = new HashMap<>(6);
        map.put("name", userPO.getNameCn());
        // 兼容多个子系统用
        map.put("roles", Arrays.asList(LoginType.YQA.code));
        map.put("avatar", "https://timgsa.baidu.com/timg?image&quality=80&size=b9999_10000&sec=1561394014552&di=17b6c1233048e5276f48309b306c7699&imgtype=0&src=http%3A%2F%2Fb-ssl.duitang.com%2Fuploads%2Fitem%2F201804%2F29%2F20180429210111_gtsnf.jpg");
        map.put("ws_auto", Ws.WS_AUTO.get());
        return MJson.ok(map);
    }

    /**
     * 获取登录用户的菜单路由。
     * @return
     */
    @ApiIgnore
    @GetMapping(AUri.USER_ROUTER)
    public MJson<Object> getRouters() {
        SPSecurityUser securityUser = AdminUtil.getUser();
        List<ModulePermissionVO> list = permissionService.findPermission(securityUser.getUserId(), securityUser.getUsername());
        List<RouterMenuVO>  routerList = PrivilegeRouterHelper.buildRouterMenu(list);
        return MJson.ok(routerList);
    }
}
