package com.sp.blinddate.module.permission;

import com.sp.blinddate.base.*;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.module.permission.vo.PrivilegeTreeVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author: jahe.lai
 * @create: 2019-09-26 18:05
 */
public class PrivilegeHelper {

    private static Logger log = LoggerFactory.getLogger(PrivilegeHelper.class);
    private static final char ZERO = '0';
    private static final char ONE = '1';


    public static List<PrivilegeTreeVO> buildPrivilegeTree(List<ModulePermissionVO> voList) {
        if (voList.isEmpty()) {
            return Collections.emptyList();
        }
        List<PrivilegeTreeVO> treeVOList = new ArrayList<>(64);
        PMenu modulePMenu;
        for (ModulePermissionVO permissionVO : voList) {
            modulePMenu = PMenu.get(permissionVO.pModule);
            if (null == modulePMenu) {
                continue;
            }
            PrivilegeTreeVO treeVO=null;
            for (PrivilegeTreeVO vo : treeVOList) {
                if (vo.getCode().equals(modulePMenu.name())) {
                    treeVO = vo;
                    break;
                }
            }
            if (null == treeVO) {
                treeVO = PrivilegeTreeVO.of(modulePMenu);
                treeVOList.add(treeVO);
            }
            treeVO.getChildren().add(PrivilegeTreeVO.of(permissionVO.pModule, permissionVO.mnodeList));
        }

        return treeVOList;
    }


    public static List<String> buildNodeList(List<ModulePermissionVO> voList) {
        if (voList.isEmpty()) {
            return Collections.emptyList();
        }
        List<String> list = new ArrayList<>(64);
        for (ModulePermissionVO permissionVO : voList) {
            for (MNode mNode : permissionVO.mnodeList) {
                list.add(permissionVO.pModule.pname + AdminConst.SEPARATOR_VERTICAL_LINE + mNode.getPname());
            }
        }
        return list;
    }

    public static void changePrivileges(Map<String, String> map) {
        if (null == map) {
            return;
        }
        for (Map.Entry<String, String> entry : map.entrySet()) {
            String nodeStr = genPrivilege(entry.getKey(), entry.getValue());
            entry.setValue(nodeStr);
        }
    }

    /**
     * 生成模块的权限表示字条串
     * @param pname	模块名称
     * @param nodes 节点列表，以"|"间隔。
     * @return String
     */
    public static String genPrivilege(String pname, String nodes) {
        if (StringUtils.isBlank(nodes)) {
            return null;
        }
        PModule pmodule = PModule.get(pname);
        if (null == pmodule) {
            return null;
        }
        String[] nodeArr = StringUtils.split(nodes, AdminConst.SEPARATOR_VERTICAL_LINE);
        // 检验
        for (String node_pname : nodeArr) {
            if (!isModuleNodes(pmodule, node_pname)) {
                log.error("PModule={} exclude {} . abort!", pname, node_pname);
                return null;
            }
        }
        return genNodePrivilege(nodeArr);
    }


    private static final char PRI[] = new char[PNode.values().length];

    private static void pri2Zero() {
        for (int i=0; i < PRI.length; i++) {
            PRI[i] = ZERO;
        }
    }
    /**
     * 权限节点转成占位字符串
     * @param nodeArr
     * @return
     */
    public static String genNodePrivilege(String[] nodeArr) {
        pri2Zero();
        // 设置拥有权限位为1
        PNode node;
        for (String node_pname : nodeArr) {
            node = PNode.get(node_pname);
            if (null == node) {
                continue;
            }
            PRI[node.getIndex()] = ONE;
        }
        return new String(PRI);
    }

    public static List<String> pnameList(PModule module, String privilege) {
        List<String> nodeList = new ArrayList<>(module.nodes.length);
        char priArr[] = privilege.toCharArray();
        String node_pname;
        for (int index=0; index<priArr.length; index++) {
            if (priArr[index] == ONE) {
                node_pname = PNode.get(index).getPname();
                for (MNode node : module.nodes) {
                    if (StringUtils.equals(node.getPname(), node_pname)) {
                        nodeList.add(node.getPname());
                        break;
                    }
                }
            }
        }
        return nodeList;
    }

    public static List<MNode> privilegeToMNode(PModule module, String privilege) {
        if (null == module || null == module.nodes) {
            return Collections.EMPTY_LIST;
        }
        List<MNode> nodeList = new ArrayList<>(module.nodes.length);
        char priArr[] = privilege.toCharArray();
        String node_pname;
        for (int index=0; index<priArr.length; index++) {
            if (priArr[index] == ONE) {
                node_pname = PNode.get(index).getPname();
                for (MNode node : module.nodes) {
                    if (StringUtils.equals(node.getPname(), node_pname)) {
                        nodeList.add(node);
                        break;
                    }
                }
            }
        }
        return nodeList;
    }

    public static boolean isModuleNodes(PModule module, String node_pname) {
        for (MNode node : module.nodes) {
            if (StringUtils.equals(node.getPname(), node_pname)) {
                return true;
            }
        }
        return false;
    }
}
