package com.sp.blinddate.module.permission;

import com.sp.blinddate.base.AUri;
import com.sp.blinddate.base.PMenu;
import com.sp.blinddate.base.PModule;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.module.permission.vo.RouterMenuMetaVO;
import com.sp.blinddate.module.permission.vo.RouterMenuVO;
import com.sp.blinddate.util.AdminUtil;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 权限生成前端路由的辅助类
 * @author: jahe.lai
 * @create: 2019-10-10 11:20
 */
public class PrivilegeRouterHelper {

    public static List<RouterMenuVO> buildRouterMenu(List<ModulePermissionVO> modulePermissionVOList) {
        if (modulePermissionVOList.isEmpty()) {
            return Collections.emptyList();
        }
        List<RouterMenuVO> routerMenuList = new ArrayList<>(32);
        PMenu modulePMenu;
        for (ModulePermissionVO permissionVO : modulePermissionVOList) {
            modulePMenu = PMenu.get(permissionVO.pModule);
            if (null == modulePMenu) {
                continue;
            }
            PMenu parentMenu = modulePMenu.parent_menu; // 查看是否有父级菜单定义
            if (null == parentMenu) {
                buildRouterMenu(routerMenuList, modulePMenu, permissionVO);
            } else {
                // 最高层菜单加入
                RouterMenuVO routerMenu = buildRouterMenu(routerMenuList, parentMenu, null);
                // 本菜单加入
                buildRouterMenu(routerMenu.getChildren(), modulePMenu, permissionVO);
            }
        }
        return routerMenuList;
    }


    private static RouterMenuVO buildRouterMenu(List<RouterMenuVO> menuList, PMenu pmenu, ModulePermissionVO permissionVO) {
        RouterMenuVO menuVO = new RouterMenuVO();
        menuVO.setName(pmenu.desc);
        // 检查父菜单是否已存在
        if (menuList.contains(menuVO)) {
            for (RouterMenuVO tm : menuList) {
                if (tm.equals(menuVO)) {
                    menuVO = tm;
                    break;
                }
            }
        } else { // 新增父菜单
            menuList.add(buildRouterMenuVO(menuVO, pmenu));
        }
        // 将module菜单加到父菜单的 Children
        if (null != permissionVO) {
            if (permissionVO.pModule.display) {
                menuVO.getChildren().add(buildChildMenuVO(permissionVO));
            }
        }
        return menuVO;
    }

    private static RouterMenuVO buildRouterMenuVO(RouterMenuVO menuVO, PMenu pmenu) {
        //一级目录需要加斜杠，不然访问 会跳转404页面
        menuVO.setPath("/" + pmenu.name());
        menuVO.setComponent("Layout");
        menuVO.setMeta(new RouterMenuMetaVO(pmenu.desc, pmenu.icon));
        return menuVO;
    }
    private static RouterMenuVO buildChildMenuVO(ModulePermissionVO vo) {
        RouterMenuVO menuVO = new RouterMenuVO();
        PModule module = vo.pModule;
        menuVO.setName(module.desc);
//        menuVO.setPath(AUri.YQA + module.pname + "/" + vo.nodeNameList.get(0));
        menuVO.setPath(AUri.YQA + module.pname);
        // TODO 公共菜单
        if (vo.pModule == PModule.PERMISSION || vo.pModule == PModule.ADMIN_USER) {
            menuVO.setComponent(module.vue);
        } else { // 项目菜单  TODO
            menuVO.setComponent(AdminUtil.getUser().getLoginType() + "/" + module.vue);
        }
        menuVO.setMeta(new RouterMenuMetaVO(module.desc, module.icon));
        return menuVO;
    }
}
