package com.sp.blinddate.module.permission.controller;


import com.sp.blinddate.module.permission.service.IAdminRoleService;
import com.sp.blinddate.module.permission.service.IPermissionService;
import com.sp.blinddate.base.AUri;
import com.sp.blinddate.module.permission.PrivilegeHelper;
import com.sp.blinddate.module.permission.entity.AdminRolePO;
import com.sp.blinddate.module.permission.entity.PermissionPO;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.module.permission.vo.PrivilegeTreeVO;
import com.sp.http.MJson;
import com.sp.http.ServiceResult;
import com.sp.http.status.BaseStatus;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jahe.lai
 * @since 2019-09-27
 */
@RestController
@ApiIgnore
public class AdminRoleController {

    @Autowired
    private IAdminRoleService adminRoleService;
    @Autowired
    IPermissionService permissionService;

    @GetMapping(AUri.ADMIN_USER_MANAGER_ROLE)
    public MJson<Object> getAllRole() {
        List<AdminRolePO> poList = adminRoleService.findAll();
        return MJson.ok(poList);
    }

    @PostMapping(AUri.ADMIN_USER_ADD_ROLE)
    public MJson<Object> add(@RequestBody AdminRolePO rolePO) {
        ServiceResult<Boolean> ret =adminRoleService.add(rolePO);
        if (StringUtils.isNotBlank(ret.getMsg())) {
            return MJson.fail(ret.getMsg());
        }
        return MJson.ok();
    }

    @PostMapping(AUri.ADMIN_USER_EDIT_ROLE)
    public MJson<Object> edit(@RequestBody AdminRolePO rolePO) {
        if (null == rolePO.getRoleId() || rolePO.getRoleId() < 1) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "ID无效");
        }
        ServiceResult<Boolean> ret =adminRoleService.updateRole(rolePO);
        if (StringUtils.isNotBlank(ret.getMsg())) {
            return MJson.fail(ret.getMsg());
        }
        return MJson.ok();
    }



    /**
     * 加载指定角色ID的权限数据
     * @param roleId
     * @param nodeList 是否只加载节点列表 List<moduleName|nodeName>
     * @return
     */
    @GetMapping(AUri.ADMIN_USER_LOAD_ROLE_PRI)
    public MJson<Object> rolePrivileges(Integer roleId, Boolean nodeList) {
        if (null == roleId || roleId.intValue() < 1) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "ID无效");
        }
        List<PermissionPO> userPOList = permissionService.getRolePrivilege(roleId);
        List<ModulePermissionVO> voList = new ArrayList<>(userPOList.size());
        for (PermissionPO po : userPOList) {
            voList.add(ModulePermissionVO.toMNode(po.getModuleName(), po.getPrivilege()));
        }
        if (nodeList) {
            return MJson.ok(PrivilegeHelper.buildNodeList(voList));
        }
        List<PrivilegeTreeVO> treeVOList = PrivilegeHelper.buildPrivilegeTree(voList);
        return MJson.ok(treeVOList);
    }
}
