package com.sp.blinddate.module.permission.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.sp.blinddate.base.AdminConst;
import com.sp.blinddate.module.permission.mapper.AdminRoleMapper;
import com.sp.blinddate.module.permission.service.IAdminRoleService;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.util.AdminUtil;
import com.sp.blinddate.module.permission.entity.AdminRolePO;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.http.ServiceResult;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jahe.lai
 * @since 2019-09-27
 */
@Service
public class AdminRoleServiceImpl extends ServiceImpl<AdminRoleMapper, AdminRolePO> implements IAdminRoleService {


    @Override
    public List<AdminRolePO> findAll() {
        SPSecurityUser user = AdminUtil.getUser();
        // 超级用户，显示全部
        if (StringUtils.equals(user.getUsername(), AdminConst.SUPER_USER)) {
            return baseMapper.selectList(Wrappers.<AdminRolePO>lambdaQuery().select(
                    AdminRolePO::getRoleId,
                    AdminRolePO::getRoleName,
                    AdminRolePO::getRoleDesc,
                    AdminRolePO::getCreateUid,
                    AdminRolePO::getEnabled,
                    AdminRolePO::getPlateform,
                    AdminRolePO::getCreateTime));
        }
        // 其它用户时，只展示自己 建立的
        return baseMapper.selectList(Wrappers.<AdminRolePO>lambdaQuery()
                .select(
                        AdminRolePO::getRoleId,
                        AdminRolePO::getRoleName,
                        AdminRolePO::getRoleDesc,
                        AdminRolePO::getCreateUid,
                        AdminRolePO::getEnabled,
                        AdminRolePO::getPlateform,
                        AdminRolePO::getCreateTime)
                .eq(AdminRolePO::getCreateUid, user.getUserId()));
    }

    @Override
    public AdminRolePO findRoleName(Integer roleId) {
        AdminRolePO rolePO = baseMapper.selectOne(Wrappers.<AdminRolePO>lambdaQuery()
                .select(AdminRolePO::getRoleName)
                .eq(AdminRolePO::getRoleId, roleId));
        return rolePO;
    }

    @Override
    public ServiceResult<Boolean> add(AdminRolePO rolePO) {
        rolePO.setCreateUid(AdminUtil.getUser().getUserId());
        rolePO.setCreateTime(new Date());
        int ret = baseMapper.insert(rolePO);
        return ret > 0 ? ServiceResult.ok() : ServiceResult.fail("写入时出现了错误");
    }


    @Override
    public ServiceResult<Boolean> updateRole(AdminRolePO rolePO) {
        int ret = baseMapper.updateById(rolePO);
        return ret > 0 ? ServiceResult.ok() : ServiceResult.fail("写入时出现了错误");
    }

}
