package com.sp.blinddate.module.permission.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.sp.blinddate.base.AdminConst;
import com.sp.blinddate.base.PModule;
import com.sp.blinddate.module.permission.mapper.PermissionMapper;
import com.sp.blinddate.module.permission.service.IAdminRoleService;
import com.sp.blinddate.module.permission.service.IPermissionService;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.module.permission.vo.UserPrivilegeVO;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.util.AdminUtil;
import com.sp.blinddate.module.permission.entity.PermissionPO;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.blinddate.module.permission.vo.PrivilegeSettingVO;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户权限明细 服务实现类
 * </p>
 *
 * @author jahe.lai
 * @since 2019-09-27
 */
@Service
public class PermissionServiceImpl extends ServiceImpl<PermissionMapper, PermissionPO> implements IPermissionService {

    @Autowired
    IAdminRoleService roleService;

    @Override
    public boolean setPrivilege(PrivilegeSettingVO settingVO) {
        if (settingVO.getUserId() > 0) {
            // 用户通过角色配置权限
            return setUserPrivilege(settingVO);
        } else if (settingVO.getRoleId() > 0) {
            // 给角色配置权限
            return setRolePrivilege(settingVO);
        }
        return true;
    }

    private boolean setUserPrivilege(PrivilegeSettingVO settingVO) {
        deletePrivilege(settingVO.getUserId(), 0);
        if (settingVO.isReset()) {
            return true;
        }
        int createUid = AdminUtil.getUser().getUserId();
        PermissionPO po;
        if (settingVO.getUserId() > 0 && settingVO.getRoleId() > 0) {
            // 用户通过角色配置权限
            po = buildPO(createUid, settingVO.getUserId(), settingVO.getRoleId(),
                    "", "");
            baseMapper.insert(po);
        } else {
            // 用户自定义配置权限
            int roleId = 0;
            for (Map.Entry<String, String> entry : settingVO.getPrivileges().entrySet()) {
                po = buildPO(createUid, settingVO.getUserId(), roleId,
                        entry.getKey(), entry.getValue());
                baseMapper.insert(po);
            }
        }
        return true;
    }

    private boolean setRolePrivilege(PrivilegeSettingVO settingVO) {
        deletePrivilege(0, settingVO.getRoleId());
        if (settingVO.isReset()) {
            return true;
        }
        int userId = 0;
        int createUid = AdminUtil.getUser().getUserId();
        for (Map.Entry<String, String> entry : settingVO.getPrivileges().entrySet()) {
            PermissionPO po = buildPO(createUid, userId, settingVO.getRoleId(),
                    entry.getKey(), entry.getValue());
            baseMapper.insert(po);
        }
        return true;
    }

    private PermissionPO buildPO(int createUid, int userId, int roleId,
                                 String module, String privilege) {
        PermissionPO po = new PermissionPO();
        po.setUserId(userId);
        po.setRoleId(roleId);
        po.setCreateUid(createUid);
        po.setModuleName(module);
        po.setPrivilege(privilege);
        po.setPlateform("");
        po.setUpdateTime(DateTime.now().toDate());
        return po;
    }

    private void deletePrivilege(int userId, int roleId) {
        if (userId > 0) {
            baseMapper.delete(Wrappers.<PermissionPO>lambdaQuery()
                    .eq(PermissionPO::getUserId, userId));
        }
        if (roleId > 0) {
            baseMapper.delete(Wrappers.<PermissionPO>lambdaQuery()
                    .eq(PermissionPO::getRoleId, roleId).eq(PermissionPO::getUserId, 0));
        }
    }

    @Override
    public UserPrivilegeVO findPrivilege(Integer userId) {
        SPSecurityUser user = AdminUtil.getUser();
        List<ModulePermissionVO> permissionVOS;
        if (user.getUserId() == userId && StringUtils.equals(AdminConst.SUPER_USER, user.getUsername())) {
            permissionVOS = new ArrayList<>(PModule.values().length);
            for (PModule pModule : PModule.values()) {
                permissionVOS.add(ModulePermissionVO.toMNode(pModule));
            }
            return UserPrivilegeVO.of(permissionVOS);
        }
        UserPrivilegeVO vo = getUserPrivilege(userId);
        permissionVOS = new ArrayList<>(vo.userPOList.size());
        for (PermissionPO po : vo.userPOList) {
            permissionVOS.add(ModulePermissionVO.toMNode(po.getModuleName(), po.getPrivilege()));
        }
        vo.setPermissionVOList(permissionVOS);
        return vo;
    }

    @Override
    public List<ModulePermissionVO> findPermission(Integer userId, String username) {
        List<ModulePermissionVO> permissionVOS;
        if (StringUtils.equals(AdminConst.SUPER_USER, username)) {
            permissionVOS = new ArrayList<>(PModule.values().length);
            for (PModule pModule : PModule.values()) {
                permissionVOS.add(ModulePermissionVO.of(pModule));
            }
        } else {
            UserPrivilegeVO vo = getUserPrivilege(userId);
            permissionVOS = new ArrayList<>(vo.userPOList.size());
            for (PermissionPO po : vo.userPOList) {
                permissionVOS.add(ModulePermissionVO.of(po.getModuleName(), po.getPrivilege()));
            }
        }
        return permissionVOS;
    }

    private UserPrivilegeVO getUserPrivilege(int userId) {
        List<PermissionPO> poList = baseMapper.selectList(Wrappers.<PermissionPO>lambdaQuery()
                .select(
                        PermissionPO::getRoleId,
                        PermissionPO::getModuleName,
                        PermissionPO::getPrivilege,
                        PermissionPO::getPlateform)
                .eq(PermissionPO::getUserId, userId));
        // 判断是否引用了角色的权限
        if (poList.size() == 1) {
            PermissionPO po = poList.get(0);
            if (po.getRoleId() > 0) {
                String roleName = roleService.findRoleName(po.getRoleId()).getRoleName();
                return UserPrivilegeVO.of(po.getRoleId(), roleName, getRolePrivilege(po.getRoleId()));
            }
        }
        return UserPrivilegeVO.ofPO(poList);
    }

    @Override
    public List<PermissionPO> getRolePrivilege(int roleId) {
        return baseMapper.selectList(Wrappers.<PermissionPO>lambdaQuery()
                .select(
                        PermissionPO::getModuleName,
                        PermissionPO::getPrivilege,
                        PermissionPO::getPlateform)
                .eq(PermissionPO::getRoleId, roleId)
                .eq(PermissionPO::getUserId, 0));
    }
}
