package com.sp.blinddate.module.permission.vo;

import com.sp.blinddate.base.MNode;
import com.sp.blinddate.base.PModule;
import com.sp.blinddate.module.permission.PrivilegeHelper;

import java.util.ArrayList;
import java.util.List;

/**
 * 一个模块的权限模型
 * 1。 数据库加载出基础数据
 * 2。 一条数据转成一个此对象，并返回列表
 * 3。可以进一步加工， 转换成需要菜单结构
 * @author: jahe.lai
 * @create: 2019-09-27 10:50
 */
public class ModulePermissionVO {

    /** 与定义常量的数据一模一样 */
    public PModule pModule;

    /** 经过页面配置的权限点(pname)集合 （来自DB） 用于权限校验 */
    public List<String> nodeNameList;

    /** 用于权限配置 */
    public List<MNode> mnodeList;

    private ModulePermissionVO() {}

    public static ModulePermissionVO of (PModule module) {
        ModulePermissionVO vo = new ModulePermissionVO();
        vo.nodeNameList = new ArrayList<>(module.nodes.length);
        vo.pModule = module;
        for (MNode mNode : module.nodes) {
            vo.nodeNameList.add(mNode.getPname());
        }
        return vo;
    }

    public static ModulePermissionVO of (String modulePName, String privilege) {
        ModulePermissionVO vo = new ModulePermissionVO();
        vo.pModule = PModule.get(modulePName);
        vo.nodeNameList = PrivilegeHelper.pnameList(vo.pModule, privilege);
        return vo;
    }


    public static ModulePermissionVO ofJWT (PModule module, List<String> nodePNameList) {
        ModulePermissionVO vo = new ModulePermissionVO();
        vo.nodeNameList = new ArrayList<>(module.nodes.length);
        vo.pModule = module;
        vo.nodeNameList.addAll(nodePNameList);
        return vo;
    }


    public static ModulePermissionVO toMNode (PModule module) {
        ModulePermissionVO vo = new ModulePermissionVO();
        vo.mnodeList = new ArrayList<>(module.nodes.length);
        vo.pModule = module;
        for (MNode mNode : module.nodes) {
            vo.mnodeList.add(mNode);
        }
        return vo;
    }
    public static ModulePermissionVO toMNode (String modulePName, String privilege) {
        ModulePermissionVO vo = new ModulePermissionVO();
        vo.pModule = PModule.get(modulePName);
        vo.mnodeList = PrivilegeHelper.privilegeToMNode(vo.pModule, privilege);
        return vo;
    }
}
