package com.sp.blinddate.module.permission.vo;

import com.sp.blinddate.base.AdminConst;
import com.sp.blinddate.base.MNode;
import com.sp.blinddate.base.PMenu;
import com.sp.blinddate.base.PModule;

import java.util.ArrayList;
import java.util.List;

/**
 * 用户的权限数据 在前端树形展示的 数据模型
 * @author: jahe.lai
 * @create: 2019-10-10 11:28
 */
public class PrivilegeTreeVO {

    /** 菜单值 / 模块值 / 节点值 */
    private String code;
    /** 菜单描述 / 模块描述 / 节点描述 */
    private String desc;

    /** 子节点 */
    private List<PrivilegeTreeVO> children;


    public static PrivilegeTreeVO of (PMenu menu) {
        PrivilegeTreeVO vo = of(menu.name(), menu.desc);
        vo.children = new ArrayList<>(menu.modules.length);
        return vo;
    }

    public static PrivilegeTreeVO of (PModule module, List<MNode> nodeList) {
        PrivilegeTreeVO vo = new PrivilegeTreeVO();
        vo.code = module.pname;
        vo.desc = module.desc;
        vo.children = new ArrayList<>(nodeList.size());
        for (MNode node : nodeList) {
            vo.children.add(of(vo.getCode()+ AdminConst.SEPARATOR_VERTICAL_LINE +node.getPname(), node.getDesc()));
        }
        return vo;
    }
    public static PrivilegeTreeVO of (String code, String desc) {
        PrivilegeTreeVO vo = new PrivilegeTreeVO();
        vo.code = code;
        vo.desc = desc;
        return vo;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    public List<PrivilegeTreeVO> getChildren() {
        return children;
    }

    public void setChildren(List<PrivilegeTreeVO> children) {
        this.children = children;
    }

    @Override
    public boolean equals(Object obj) {
        if (null == obj) {
            return false;
        }
        PrivilegeTreeVO objVO = (PrivilegeTreeVO) obj;
        if (this.code.equals(objVO.getCode())) {
            return true;
        }
        return false;
    }
}
