package com.sp.blinddate.module.user.controller;


import com.sp.blinddate.base.AUri;
import com.sp.blinddate.base.AdminConst;
import com.sp.blinddate.module.permission.PrivilegeHelper;
import com.sp.blinddate.module.permission.service.IPermissionService;
import com.sp.blinddate.module.user.service.IAdminUserService;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.util.AdminUtil;
import com.sp.blinddate.module.permission.vo.PrivilegeSettingVO;
import com.sp.blinddate.module.permission.vo.PrivilegeTreeVO;
import com.sp.blinddate.module.permission.vo.UserPrivilegeVO;
import com.sp.blinddate.module.user.entity.AdminUserPO;
import com.sp.http.MJson;
import com.sp.http.ServiceResult;
import com.sp.http.status.BaseStatus;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.*;

/**
 * <p>
 * 运营后台用户表 前端控制器
 * </p>
 *
 * @author jahe.lai
 * @since 2019-09-26
 */
@RestController
@ApiIgnore
public class AdminUserController {

    @Autowired
    private IAdminUserService adminUserService;
    @Autowired
    IPermissionService permissionService;


    @PostMapping(AUri.ADMIN_USER_ADD)
    public MJson<Object> add(@RequestBody AdminUserPO userPO) {
        ServiceResult<Boolean> ret =adminUserService.addUser(userPO);
        if (StringUtils.isNotBlank(ret.getMsg())) {
            return MJson.fail(ret.getMsg());
        }
        return MJson.ok();
    }

    /**
     * 修改用户的信息数据，密码除外
     * @param userPO
     * @return
     */
    @PostMapping(AUri.ADMIN_USER_EDIT)
    public MJson<Object> edit(@RequestBody AdminUserPO userPO) {
        if (null == userPO.getId() || userPO.getId() < 1) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "ID无效");
        }
        // 标记删除时，同时也锁定。
        if (userPO.getDelFlag()) {
            userPO.setLockFlag(Boolean.TRUE);
        }
        ServiceResult<Boolean> ret =adminUserService.updateUser(userPO);
        if (StringUtils.isNotBlank(ret.getMsg())) {
            return MJson.fail(ret.getMsg());
        }
        return MJson.ok();
    }

    /**
     * 修改密码
     * @param pre_password
     * @param password
     * @return
     */
    @PostMapping(AUri.ADMIN_USER_UPDATE_PSW)
    public MJson<Object> updatePsw(Integer id, String pre_password, String password) {
        if (null == id || id < 1) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "ID无效");
        }
        if (StringUtils.isBlank(pre_password) || StringUtils.isBlank(password)) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "提供的旧/新密码无效");
        }
        ServiceResult<Boolean> ret = adminUserService.updatePass(id, pre_password, password);
        if (StringUtils.isNotBlank(ret.getMsg())) {
            return MJson.fail(ret.getMsg());
        }
        return MJson.ok();
    }


    /**
     * 默认加载当前登录用户的权限数据
     * @param userId
     * @param nodeList 是否只加载节点列表 List<moduleName|nodeName>
     * @return
     */
    @GetMapping(AUri.ADMIN_USER_LOAD_USER_PRI)
    public MJson<Object> userPrivileges(Integer userId, Boolean nodeList) {
        if (null == userId || userId.intValue() < 1) {
            userId = AdminUtil.getUser().getUserId();
        }
        UserPrivilegeVO userPrivilegeVO = permissionService.findPrivilege(userId);
        Map<String, Object> map = new HashMap<>(6);
        map.put("roleId", userPrivilegeVO.getRoleId());
        map.put("roleName", userPrivilegeVO.getRoleName());
        if (nodeList) {
            List<String> treeData = PrivilegeHelper.buildNodeList(userPrivilegeVO.getPermissionVOList());
            map.put("treeData", treeData);
        } else {
            List<PrivilegeTreeVO> treeVOList = PrivilegeHelper.buildPrivilegeTree(
                    userPrivilegeVO.getPermissionVOList());
            map.put("treeData", treeVOList);
        }
        return MJson.ok(map);
    }

    @PostMapping(AUri.ADMIN_USER_DELETE)
    public MJson<Object> deleteUser(Integer id) {
        if (null == id || id < 1) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "ID无效");
        }
        boolean ret = adminUserService.deleteUser(id);
        return ret ? MJson.ok() : MJson.fail();
    }

    /**
     * 返回所有的运营用户数据。
     * @return
     */
    @GetMapping(AUri.ADMIN_USER_MANAGER)
    public MJson<Object> getAllAdminUser() {
        List<AdminUserPO> adminUserPOList = adminUserService.findAll();
        return MJson.ok(adminUserPOList);
    }


    @PostMapping(AUri.ADMIN_USER_SETTING_PRIVILEGE)
    public MJson<Object> setPrivilege(@RequestBody PrivilegeSettingVO settingVO) {
        settingVO.setRoleId(1);
        SPSecurityUser loginUser = AdminUtil.getUser();
        if (settingVO.getUserId()== loginUser.getUserId() &&
                AdminConst.SUPER_USER.equals(loginUser.getUsername())) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "SUPER不需要配置权限。");
        }
        if (settingVO.getRoleId() < 1 && settingVO.getUserId() < 1) {
            return MJson.resp(BaseStatus.PARAM_INVALID, "ID无效");
        }
        if (settingVO.getRoleId() == 0 && settingVO.getUserId() > 0) {
            if (settingVO.isReset()==false && (null == settingVO.getPrivileges() || settingVO.getPrivileges().isEmpty())) {
                return MJson.resp(BaseStatus.PARAM_INVALID, "需要给用户提供权限数据");
            }
        }
        if (settingVO.getRoleId() > 0 && settingVO.getUserId() == 0) {
            if (settingVO.isReset()==false && (null == settingVO.getPrivileges() || settingVO.getPrivileges().isEmpty())) {
                return MJson.resp(BaseStatus.PARAM_INVALID, "需要给角色提供权限数据");
            }
        }
        PrivilegeHelper.changePrivileges(settingVO.getPrivileges());
        boolean ret = permissionService.setPrivilege(settingVO);
        return ret ? MJson.ok() : MJson.fail();
    }


}
