package com.sp.blinddate.module.user.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.sp.blinddate.base.AdminConst;
import com.sp.blinddate.exception.BaseException;
import com.sp.blinddate.module.permission.service.IPermissionService;
import com.sp.blinddate.module.user.entity.AdminUserPO;
import com.sp.blinddate.module.user.mapper.AdminUserMapper;
import com.sp.blinddate.module.user.service.IAdminUserService;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.util.AdminUtil;
import com.sp.blinddate.util.JwtUtil;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.http.ServiceResult;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.*;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 运营后台用户表 服务实现类
 * </p>
 *
 * @author jahe.lai
 * @since 2019-09-26
 */
@Service
public class AdminUserServiceImpl extends ServiceImpl<AdminUserMapper, AdminUserPO> implements IAdminUserService {


    @Autowired
    private AuthenticationManager authenticationManager;
    @Autowired
    private IPermissionService permissionUserService;

    @Override
    public String checkLogin(String username, String password) {
        Authentication authentication = null;
        try {
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername()去验证用户名和密码，
            // 如果正确，则存储该用户名密码到security 的 context中
            authentication = authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(username, password));
        } catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new BaseException("用户名或密码错误", 402);
            } else if (e instanceof DisabledException) {
                throw new BaseException("账户被禁用", 402);
            } else if (e instanceof AccountExpiredException) {
                throw new BaseException("账户过期无法验证", 402);
            } else {
                throw new BaseException("账户被锁定,无法登录", 402);
            }
        }
        //存储认证信息
        SecurityContextHolder.getContext().setAuthentication(authentication);
        //生成token
        SPSecurityUser userDetail = (SPSecurityUser) authentication.getPrincipal();
        String token = JwtUtil.generateToken(userDetail);
        System.out.println(token);
        return token;
    }

    @Override
    public AdminUserPO findByUserName(String username) {
        AdminUserPO adminUserPO = baseMapper.selectOne(Wrappers.<AdminUserPO>lambdaQuery()
                .select(AdminUserPO::getId, AdminUserPO::getAccount, AdminUserPO::getNameCn,
                        AdminUserPO::getPassword, AdminUserPO::getLockFlag, AdminUserPO::getDelFlag)
                .eq(AdminUserPO::getAccount, username));
        return adminUserPO;
    }

    @Override
    public AdminUserPO findInfo(Integer userId) {
        AdminUserPO adminUserPO = baseMapper.selectOne(Wrappers.<AdminUserPO>lambdaQuery()
                .select(AdminUserPO::getId,
                        AdminUserPO::getNameCn,
                        AdminUserPO::getLockFlag,
                        AdminUserPO::getDelFlag)
                .eq(AdminUserPO::getId, userId));
        return adminUserPO;
    }

    private AdminUserPO find(Integer userId) {
        AdminUserPO adminUserPO = baseMapper.selectOne(Wrappers.<AdminUserPO>lambdaQuery()
                .select(AdminUserPO::getId,
                        AdminUserPO::getAccount,
                        AdminUserPO::getNameCn,
                        AdminUserPO::getNameEn,
                        AdminUserPO::getCreateTime,
                        AdminUserPO::getLockFlag,
                        AdminUserPO::getDelFlag)
                .eq(AdminUserPO::getId, userId));
        return adminUserPO;
    }

    private AdminUserPO findAccountPsw(Integer id) {
        AdminUserPO adminUserPO = baseMapper.selectOne(Wrappers.<AdminUserPO>lambdaQuery()
                .select(AdminUserPO::getAccount, AdminUserPO::getPassword)
                .eq(AdminUserPO::getId, id));
        return adminUserPO;
    }
    private AdminUserPO findAccountId(String account) {
        AdminUserPO adminUserPO = baseMapper.selectOne(Wrappers.<AdminUserPO>lambdaQuery()
                .select(AdminUserPO::getId)
                .eq(AdminUserPO::getAccount, account));
        return adminUserPO;
    }

    @Override
    public ServiceResult<Boolean> addUser(AdminUserPO userPO) {
        AdminUserPO po = findAccountId(userPO.getAccount());
        if (null != po && po.getId() > 0) {
            return ServiceResult.fail("数据库中已存在此帐号记录");
        }
        userPO.setPassword(AdminUtil.encode(userPO.getPassword()));
        userPO.setGenUid(AdminUtil.getUser().getUserId());
        userPO.setCreateTime(new Date());
        int ret = baseMapper.insert(userPO);
        return ret > 0 ? ServiceResult.ok() : ServiceResult.fail("写入时出现了错误");
    }

    @Override
    public ServiceResult<Boolean> updateUser(AdminUserPO userPO) {
        AdminUserPO po = findAccountPsw(userPO.getId());
        if (!StringUtils.equals(userPO.getAccount(), po.getAccount()) ) { // 帐号有变动
            AdminUserPO idPO = findAccountId(userPO.getAccount());
            if (null != idPO && idPO.getId() > 0) {
                return ServiceResult.fail(String.format("帐号 【%s】 已存在，请更改后重试。",
                        userPO.getAccount()));
            }
        }
        userPO.setPassword(po.getPassword());
        int ret = baseMapper.updateById(userPO);
        return ret > 0 ? ServiceResult.ok() : ServiceResult.fail("写入时出现了错误");
    }

    @Override
    public boolean deleteUser(Integer userId) {
        int ret = baseMapper.deleteById(userId);
        return  ret > 0;
    }

    @Override
    public ServiceResult<Boolean> updatePass(Integer userId, String pre_password, String password) {
        AdminUserPO po = findAccountPsw(userId);
        if (null == po) {
            return ServiceResult.fail("未找到对应用户数据");
        }
        if ( !AdminUtil.validatePass(pre_password, po.getPassword())) {
            return ServiceResult.fail("旧密码不匹配，请确认。");
        }
        password = AdminUtil.encode(password);
        int ret = baseMapper.updateById(new AdminUserPO().setPassword(password).setId(userId));
        return ret > 0 ? ServiceResult.ok() : ServiceResult.fail("写入时出现了错误");
    }

    @Override
    public List<ModulePermissionVO> findPermission(Integer userId, String username) {
        return permissionUserService.findPermission(userId, username);
    }

    @Override
    public List<AdminUserPO> findAll() {
        SPSecurityUser user = AdminUtil.getUser();
        // 超级用户，显示全部
        if (StringUtils.equals(user.getUsername(), AdminConst.SUPER_USER)) {
            return baseMapper.selectList(Wrappers.<AdminUserPO>lambdaQuery().select(
                    AdminUserPO::getId,
                    AdminUserPO::getAccount,
                    AdminUserPO::getNameCn,
                    AdminUserPO::getNameEn,
                    AdminUserPO::getCreateTime,
                    AdminUserPO::getLockFlag,
                    AdminUserPO::getDelFlag));
        }
        // 其它用户时，只展示自己 和 建立的用户
        List<AdminUserPO> userPOList = baseMapper.selectList(Wrappers.<AdminUserPO>lambdaQuery()
                .select(
                        AdminUserPO::getId,
                        AdminUserPO::getAccount,
                        AdminUserPO::getNameCn,
                        AdminUserPO::getNameEn,
                        AdminUserPO::getCreateTime,
                        AdminUserPO::getLockFlag,
                        AdminUserPO::getDelFlag)
                .eq(AdminUserPO::getGenUid, user.getUserId()));
        userPOList.add(find(user.getUserId()));
        return userPOList;
    }
}
