package com.sp.blinddate.security;

import com.sp.blinddate.base.LoginType;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.module.user.entity.AdminUserPO;
import com.sp.blinddate.module.user.service.IAdminUserService;
import com.sp.utils.JsonUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author: jahe.lai
 * @create: 2019-09-23 17:04
 */
@Service
public class SPUserDetailsService implements UserDetailsService {


    private static Logger log = LoggerFactory.getLogger(SPUserDetailsService.class);

    @Autowired
    private IAdminUserService adminUserService;

    /**
     * 用户名密码登录
     *
     * @param username
     * @return
     */
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        AdminUserPO user = adminUserService.findByUserName(username);
        if (null == user) {
            log.info("登录用户：" + username + " 不存在.");
            throw new UsernameNotFoundException("登录用户：" + username + " 不存在");
        }
        SPSecurityUser securityUser = new SPSecurityUser(user.getId(), username, user.getPassword(),
                LoginType.YQA, user.getLockFlag());
        List<ModulePermissionVO> permissionVOList = adminUserService.findPermission(user.getId(), user.getAccount());
        securityUser.setPermissionList(permissionVOList);
        log.warn("权限={}", JsonUtil.toStr(permissionVOList));
        return securityUser;
    }
}
