package com.sp.blinddate.security.filter;

import com.sp.blinddate.base.AUri;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.base.AdminCache;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.util.JwtUtil;
import com.sp.blinddate.util.HttpHelper;
import com.sp.http.MJson;
import com.sp.http.status.IStatus;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @author: jahe.lai
 * @create: 2019-09-27 17:45
 */
@Component
public class MySecurityPrivilegeFilter extends OncePerRequestFilter {

    private static Logger log = LoggerFactory.getLogger(MySecurityPrivilegeFilter.class);

    @Autowired
    private JwtUtil jwtUtil;
    @Autowired
    AdminCache adminCache;

    // 权限节点位置
    private static final int NODE_INDEX = AUri.YQA.length() > 1 ? 2 : 1;


    /**
     * 在此实现此访问请求的权限验证，没有访问权限响应提示并终止请求。
     */
    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        String uri = request.getRequestURI();
        String[] uriArr = StringUtils.split(uri, "/");
        if (uriArr.length < (NODE_INDEX + 1)) {	// 不符合权限校验的结构
            doFilter(request, response, filterChain);
            return;
        }
        SPSecurityUser securityUser = jwtUtil.getUserFromToken(request);
        if (securityUser == null) {	// 未登录、登录超时
//            doFilter(request, response, filterChain);
            HttpHelper.ok(MJson.resp(IStatus.HTTP_UNAUTHORIZED, "您的登录超了有效时长。"), response);
            return;
        }

        boolean check = checkPrivilege(uriArr, securityUser);
        if (check) {
            doFilter(request, response, filterChain);
            return;
        }

        // 权限校验不通过
        HttpHelper.ok(MJson.resp(IStatus.HTTP_FORBIDDEN,
                "请求访问:" + uri + "接口需要授权."),response);
    }

    /**
     * 检查是否有权限执行此URI的授权。
     * @param uriArr String[]
     * @param securityUser SPSecurityUser
     * @return boolean
     */
    private boolean checkPrivilege(String[] uriArr, SPSecurityUser securityUser) {
        String module_name = uriArr[NODE_INDEX-1];
        String node_name = uriArr[NODE_INDEX];
        List<ModulePermissionVO> plist = adminCache.getPermission(securityUser.getUsername());
        boolean check = false;
        for (ModulePermissionVO permissionVO : plist) {
            if (StringUtils.equals(permissionVO.pModule.pname, module_name)) {
                if (permissionVO.nodeNameList.contains(node_name)) {
                    check = true;
                    break;
                }
            }
        } // end for
        return check;
    }

    @Override
    protected boolean shouldNotFilter(HttpServletRequest request) throws ServletException {
        String uri = request.getRequestURI();
        if(uri.contains("webjars")){
            return true;
        }
        return super.shouldNotFilter(request);
    }
}
