package com.sp.blinddate.security.vo;

import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import com.sp.blinddate.base.LoginType;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * 权限验证的数据模型
 * @author: jahe.lai
 * @create: 2019-09-24 10:36
 */
@Data
@Accessors(chain = true)
public class SPSecurityUser implements UserDetails {

    private static final long serialVersionUID = 1L;


    private String loginType = "";
    private Integer userId;
    private String username;
    private String password;
    private boolean nonlock = true;
    private Collection<? extends GrantedAuthority> authorities = Collections.EMPTY_LIST;
    List<ModulePermissionVO> permissionList;


    public SPSecurityUser(Integer userId, String username, String password,
                         LoginType loginType) {
        this(userId, username, password, loginType, true);
    }

    public SPSecurityUser(Integer userId, String username, String password,
                          LoginType loginType, boolean lock) {
        this.userId = userId;
        this.username = username;
        this.password = password;
        this.loginType = loginType.code;
        this.nonlock = !lock;
    }

    public List<ModulePermissionVO> getPermissionList() {
        return permissionList;
    }

    public void setPermissionList(List<ModulePermissionVO> permissionList) {
        this.permissionList = permissionList;
    }

    /**
     * 返回分配给用户的角色列表
     * @return
     */
    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return authorities;
    }

    /**
     * 账户是否未过期,过期无法验证
     * @return
     */
    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    /**
     * 指定用户是否解锁,锁定的用户无法进行身份验证
     * @return
     */
    @Override
    public boolean isAccountNonLocked() {
        return nonlock;
    }

    /**
     * 指示是否已过期的用户的凭据(密码),过期的凭据防止认证
     * @return
     */
    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    /**
     * 是否可用 ,禁用的用户不能身份验证
     * @return
     */
    @Override
    public boolean isEnabled() {
        return true;
    }
}
