package com.sp.blinddate.websocket;

import com.sp.blinddate.util.JwtUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.support.HttpSessionHandshakeInterceptor;

import java.util.Map;

/**
 * @author: jahe.lai
 * @create: 2019-10-15 16:51
 */
public class WebSocketHandshakeInterceptor extends HttpSessionHandshakeInterceptor {

    private static Logger logger = LoggerFactory.getLogger(WebSocketHandshakeInterceptor.class);

    @Override
    public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler,
                                   Map<String, Object> attributes) throws Exception {
        ///获取请求参数，首先我们要获取HttpServletRequest对象才能获取请求参数；当ServerHttpRequset的层次结构打开后其子类可以获取到我们想要的http对象，那么就简单了。
        //我这里是把获取的请求数据绑定到session的map对象中（attributes）
        ServletServerHttpRequest servletRequest = (ServletServerHttpRequest) request;
        String token = servletRequest.getServletRequest().getParameter("token");
//        logger.warn("Before Handshake {}", token);
        String userName = JwtUtil.getUsernameFromToken(token);
        if(userName == null){
            userName = "WEBSOCKET_USERNAME_IS_NULL";
        }
        attributes.put("username", userName);
        logger.warn("Before Handshake {}", userName);
        return super.beforeHandshake(request, response, wsHandler, attributes);
    }

    @Override
    public void afterHandshake(ServerHttpRequest request,
                               ServerHttpResponse response, WebSocketHandler wsHandler,
                               Exception ex) {
//        logger.info("After Handshake.");
        super.afterHandshake(request, response, wsHandler, ex);
    }
}
