package com.sp.blinddate.websocket;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

import com.sp.blinddate.util.AdminUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.socket.BinaryMessage;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

import com.google.common.collect.Maps;

public class Ws {

	private static Logger logger = LoggerFactory.getLogger(Ws.class);

	/** 自动开启连接服务 */
	public static final AtomicBoolean WS_AUTO = new AtomicBoolean(false);
	public static final AtomicReference<String> WS_MSG = new AtomicReference<String>();
    /** WebSocket的会话 key=登录用户的帐号 */
    private static final Map<String, WebSocketSession> WS_SESSION = Maps.newConcurrentMap();

    protected static void addOnline(WebSocketSession session) {
        String username = getUserName(session);
    	WS_SESSION.put(username, session);
        logger.warn("online={} {}", username, WS_SESSION.size());
        if (WS_AUTO.get() && StringUtils.isNotBlank(WS_MSG.get())) {
        	sendMsg(session, WS_MSG.get());
        }
    }

    protected static String getUserName(WebSocketSession session) {
        return session.getAttributes().get("username").toString();
    }

    protected static void remove(WebSocketSession session) {
        String username = null;
       for (Map.Entry<String, WebSocketSession> entry : WS_SESSION.entrySet()) {
           if (entry.getValue().getId().equals(session.getId())) {
               username = entry.getKey();
               break;
           }
       }
       if (StringUtils.isNotBlank(username)) {
           WS_SESSION.remove(username);
           logger.warn("removed:{} online={}", username, WS_SESSION.size());
       }
    }

    public static Set<String> onlineUsers() {
    	return WS_SESSION.keySet();
    }

    public static void broadcastMsg(String message, boolean error) {
        Iterator<WebSocketSession> it= WS_SESSION.values().iterator();
        while(it.hasNext()) {
        	if (error) {
        		sendErrMsg(it.next(), message);
        	} else {
        		sendMsg(it.next(), message);
        	}
        }
    }

    public static void sendMsg(WebSocketSession session, String message) {
        if (!session.isOpen() || StringUtils.isBlank(message)) {
            return;
        }
        if (message.indexOf("{") == -1) {
            message = WsMessage.message(IWsProto.WS_MSG, message);
        }
        try {
            session.sendMessage(new TextMessage(message));
        } catch (IOException e) {
            logger.error("send err", e);
        }
    }

    public static void sendErrMsg(String message) {
    	WebSocketSession socketSession = WS_SESSION.get(AdminUtil.getUser().getUsername());
        if (null != socketSession && socketSession.isOpen()) {
            sendErrMsg(socketSession, message);
        }
    }

    public static void sendErrMsg(WebSocketSession session, String message) {
        if (!session.isOpen() || StringUtils.isBlank(message)) {
            return;
        }
        if (message.indexOf("{") == -1) {
            message = WsMessage.message(IWsProto.WS_ERR_MSG, message);
        }
        try {
            session.sendMessage(new TextMessage(message));
        } catch (IOException e) {
            logger.error("send err", e);
        }
    }
    public static void sendMsg(String message) {
        WebSocketSession socketSession = WS_SESSION.get(AdminUtil.getUser().getUsername());
        if (null != socketSession && socketSession.isOpen()) {
            sendMsg(socketSession, message);
        }
    }

    /**
     * 发送消息, 并告诉客户端使用结束,可以关闭.
     * @param message
     */
    public static void sendDoneMsg(String message) {
        sendMsg(WsMessage.message(IWsProto.WS_DONE, message));
    }
    public static void sendDoneMsg(WebSocketSession session, String message) {
        sendMsg(session, WsMessage.message(IWsProto.WS_DONE, message));
    }

    public void sendImg(WebSocketSession session,String fileName){
        FileInputStream input = null;
        try {
            File file=new File("D:\\images\\"+fileName);
            if (file.exists()) {
                input = new FileInputStream(file);
                byte bytes[] = new byte[(int) file.length()];
                input.read(bytes);
                BinaryMessage byteMessage=new BinaryMessage(bytes);
                session.sendMessage(byteMessage);
            }
        } catch (Exception e) {
            logger.error("", e);
        } finally {
            if (null != input) {
                try {
                    input.close();
                } catch (IOException e) {
                }
            }
        }// finally
    }

}
