package com.sp.blinddate.apidoc;

import com.sp.http.status.BaseStatus;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.web.bind.annotation.RequestMethod;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.ResponseMessageBuilder;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.service.Parameter;
import springfox.documentation.service.ResponseMessage;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

@Configuration
@EnableSwagger2
//@Profile("dev")
public class SwaggerConfig {

	@Value("${spring.profiles.active}")
	private String activeProfile;

	@Bean
	public Docket customImplementation(){
		List<Parameter> parameters = new ArrayList<>(2);
		Parameter userAgentParameter = new ParameterBuilder()
				.parameterType("header")
				.name("My-User-Agent")
				.defaultValue("Swagger#Swagger-H5#100#100")
				.modelRef(new ModelRef("string"))
				.description("标识用户渠道，设备号，api版本号，app版本号")
				.required(true)
				.build();
		parameters.add(userAgentParameter);

		Parameter tokenParameter = new ParameterBuilder()
				.parameterType("header")
				.name("My-Token")
				.defaultValue("")
				.description("用户token")
				.modelRef(new ModelRef("string"))
				.required(false)
				.build();
		parameters.add(tokenParameter);

		BaseStatus[] statuses = BaseStatus.values();
		List<ResponseMessage> responseMessages = new ArrayList<>(statuses.length);
		for(BaseStatus status : statuses) {
			ResponseMessage message = new ResponseMessageBuilder()
			.code(status.getCode())
			.message(status.getMsg())
			.build();
			responseMessages.add(message);
		}


		Docket docket = new Docket(DocumentationType.SWAGGER_2);
		docket.select().paths(PathSelectors.regex("^/.*")).build();
		docket.apiInfo(apiInfo())
			  .useDefaultResponseMessages(false)
			  .globalResponseMessage(RequestMethod.GET, responseMessages)
			  .globalResponseMessage(RequestMethod.POST, responseMessages)
			  .globalResponseMessage(RequestMethod.PUT, responseMessages)
			  .globalResponseMessage(RequestMethod.DELETE, responseMessages)
			  .globalOperationParameters(parameters);
		return docket;
	}

	private ApiInfo apiInfo() {
		StringBuilder descBuf = new StringBuilder(1024);
		PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
		Resource[] resources;
		try {
			resources = resolver.getResources("classpath*:doc/readme.txt");
			if(resources!=null && resources.length>0) {
				Resource resource = resources[0];
				try (BufferedReader br = new BufferedReader(new InputStreamReader(resource.getInputStream()))) {
					String line = null;
					while( (line=br.readLine()) != null ) {
						descBuf.append("<p>").append(line).append("</p><br>");
					}
				} catch (Exception e) {e.printStackTrace();}
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

        ApiInfo apiInfo = new ApiInfoBuilder()
        		.title("子女帮帮小程序-接口管理")
				.contact(new Contact("shiyi","https://www.xsbanruo.com/","ishiyi@163.com"))
        		.description(descBuf.toString())
        		.build();
        return apiInfo;
    }
}
