package com.sp.blinddate.module.chlidinfo.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sp.RedisServer;
import com.sp.blinddate.common.BaseReviewIenum;
import com.sp.blinddate.module.chlidinfo.keyprefix.UserInfoKeyPrefix;
import com.sp.blinddate.module.chlidinfo.mapper.ChildInfoMapper;
import com.sp.blinddate.module.chlidinfo.param.ChildInfoParam;
import com.sp.blinddate.module.chlidinfo.service.ChildInfoService;
import com.sp.blinddate.module.chlidinfo.vo.ChildInfoVO;
import com.sp.blinddate.module.chlidinfo.vo.UserDetailInfoVO;
import com.sp.blinddate.module.recommand.service.RecommandService;
import com.sp.blinddate.module.recommand.vo.RecommandVO;
import com.sp.blinddate.module.spouse.entity.SpouseRequest;
import com.sp.blinddate.module.spouse.service.ISpouseRequestService;
import com.sp.blinddate.module.user.entity.UserChildInfo;
import com.sp.blinddate.module.user.param.UserFullInfoParam;
import com.sp.blinddate.module.user.service.IUserChildInfoService;
import com.sp.blinddate.module.user.service.IUserChildPictureService;
import com.sp.utils.PageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Descrption
 * @Author: 十一
 * @Date: 2019/11/25 17:49
 **/
@Service
public class ChildInfoServiceImpl implements ChildInfoService {
    private static final Logger LOG = LoggerFactory.getLogger(ChildInfoServiceImpl.class);

    @Autowired
    private ChildInfoMapper childInfoMapper;

    @Autowired
    private RedisServer redisServer;

    @Autowired
    private IUserChildInfoService userChildInfoService;

    @Autowired
    private IUserChildPictureService userChildPictureService;

    @Autowired
    private ISpouseRequestService spouseRequestService;

    @Autowired
    private RecommandService recommandService;


    /**
     *功能描述 审核用户信息
     * @author 十一
     * @date 2019/11/27 16:31
     */
    @Override
    public void verifyUserInfo(Long userInfoId, BaseReviewIenum status) {
        UpdateWrapper<UserChildInfo> uw = new UpdateWrapper<>();
        uw.lambda().set(UserChildInfo::getStatus, status)
                .eq(UserChildInfo::getId, userInfoId);
        boolean update = userChildInfoService.update(uw);
        if(update){
            UserChildInfo info = userChildInfoService.getById(userInfoId);
            String key = info.getResidenceProvince()+":"+info.getResidenceCity()+":"+info.getGender().getNum();
            //如果是审核通过，将当前的用户信息放入到用户信息集合中,目前只能匹配到用户所在的市
            if(status.equals(BaseReviewIenum.Pass)){
                Long score = System.currentTimeMillis();
                redisServer.zadd(UserInfoKeyPrefix.userPool, key,info.getId()+"", score.doubleValue());
                LOG.info("审核成功，放入用户池成功：" + key);
                //如果是下架，那么将这个人从资料库中移除
            }else if(status.equals(BaseReviewIenum.Obtained)){
                Boolean result = redisServer.zream(UserInfoKeyPrefix.userPool, key, info.getId() + "");
                if(result){
                    LOG.info("下架成功");
                }
            }
        }
    }

    /**
     *功能描述 获取择偶详情
     * @author 十一
     * @date 2019/11/25 19:52
     */
    @Override
    public UserDetailInfoVO getDetailInfo(Long infoId) {
        List<String> pictures = userChildPictureService.getUserChildPictures(infoId);
        UserChildInfo info = userChildInfoService.getById(infoId);
        SpouseRequest request = spouseRequestService.getUserRequsetByUserInfoId(infoId);
        UserDetailInfoVO vo = new UserDetailInfoVO();
        vo.setPictures(pictures);
        vo.setInfo(info);
        vo.setSpouseRequest(request);
        return vo;
    }

    /**
     *功能描述 查询相信资料列表
     * @author 十一
     * @date 2019/11/25 17:58
     */
    @Override
    public PageInfo<ChildInfoVO> getVerifyList(ChildInfoParam param) {
        Integer currentPage = PageUtil.verifyCurrentPage(param.getPageNum());
        PageHelper.startPage(currentPage, param.getPageSize());
        List<ChildInfoVO> list =  childInfoMapper.getVerifyList(param);
        return new PageInfo<>(list);
    }

    /**
     *功能描述 根据用户的id查询出用户子女信息
     * @author 十一
     * @date 2019/12/16 16:20
     */
    @Override
    public List<RecommandVO> getUserInfosByUserId(Long userId) {
        List<Long> userInfos = userChildInfoService.getUserChildInfoIdsByCondition(userId);
        return recommandService.getRecommandListByInfoIds(null, userInfos);
    }

    /**
     *功能描述 获取用户信息的编辑数据
     * @author 十一
     * @date 2019/12/2 17:45
     */
    @Override
    public UserFullInfoParam getUserEditInfo(Long userInfoId) {
        UserChildInfo info = userChildInfoService.getById(userInfoId);
        SpouseRequest req = spouseRequestService.getById(userInfoId);
        List<String> pics = userChildPictureService.getUserChildPictures(userInfoId);
        UserFullInfoParam param = new UserFullInfoParam();
        param.setUserInfoId(userInfoId);
        param.setCar(info.getCar());
        param.setCarrer(info.getCarrer());
        param.setCensusProvince(info.getCensusProvince());
        param.setCensusCity(info.getCensusCity());
        param.setCoverImg(info.getCoverImg());
        param.setDateOfBirth(info.getDateOfBirth());
        param.setDes(info.getDes());
        param.setGender(info.getGender());
        param.setHeight(info.getHeight());
        param.setHouse(info.getHouse());
        param.setImages(pics);
        param.setWechat(info.getWechat());
        param.setPhone(info.getPhone());
        param.setResidenceProvince(info.getResidenceProvince());
        param.setResidenceCity(info.getResidenceCity());
        param.setRelation(info.getRelation());
        param.setIncome(info.getIncome());
        param.setMaritalStatus(info.getMaritalStatus());
        param.setEducation(info.getEducation());
        param.setExpAge(req.getAge());
        param.setExpEducation(req.getEducation());
        param.setExpCensusProvince(req.getCensusProvince());
        param.setExpCensusCity(req.getCensusCity());
        param.setExpDes(req.getDes());
        param.setExpheight(req.getHeight());
        param.setExpHouse(req.getHouse());
        param.setExpIncome(req.getIncome());
        return param;
    }
}
