package com.sp.blinddate.module.likechildinfo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sp.blinddate.base.status.VIPEnum;
import com.sp.blinddate.common.BaseLikeEnum;
import com.sp.blinddate.exception.AuthorityException;
import com.sp.blinddate.module.likechildinfo.entity.UserLikeChildDetail;
import com.sp.blinddate.module.likechildinfo.mapper.UserLikeChildDetailMapper;
import com.sp.blinddate.module.likechildinfo.service.IUserLikeChildDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.blinddate.module.likechildinfo.vo.UserLikeChildVO;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.service.IUserChildInfoService;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.constant.PageConst;
import com.sp.utils.IdGenerateUtil;
import com.sp.utils.PageUtil;
import net.bytebuddy.asm.Advice;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户喜欢子女的详情表 服务实现类
 * </p>
 *
 * @author 十一
 * @since 2019-11-20
 */
@Service
public class UserLikeChildDetailServiceImpl extends ServiceImpl<UserLikeChildDetailMapper, UserLikeChildDetail> implements IUserLikeChildDetailService {

    @Autowired
    private UserLikeChildDetailMapper userLikeChildDetailMapper;

    @Autowired
    private IUserChildInfoService userChildInfoService;

    @Autowired
    private IUserService userService;

    /**
     *功能描述 查询自己喜欢过的子女的缩略信息
     * @author 十一
     * @date 2019/11/21 10:42
     */
    @Override
    public List<String> getLikedChildListByThumbnail(Long id) {
        QueryWrapper<UserLikeChildDetail> qw = new QueryWrapper<>();
        qw.lambda().eq(UserLikeChildDetail::getUserId, id)
                    .select(UserLikeChildDetail::getLikeUserMarriageNumber)
                    .orderByDesc(UserLikeChildDetail::getCreateTime);
        IPage<UserLikeChildDetail> page = new Page<>(1,3);
        IPage<UserLikeChildDetail> ipage = userLikeChildDetailMapper.selectPage(page, qw);
        List<UserLikeChildDetail> list = ipage.getRecords();
        if(list.isEmpty()){
            return Collections.emptyList();
        }
        //查询出所有的姻缘号
        List<String> marriages = list.stream().map(UserLikeChildDetail::getLikeUserMarriageNumber).collect(Collectors.toList());
        //根据姻缘号查询出用户列表
        List<String> headImgs = userChildInfoService.getUserCoverImgByMarrages(marriages);
        return headImgs;
    }

    /**
     *功能描述 查询用户被喜欢的次数
     * @author 十一
     * @date 2019/12/16 15:48
     */
    @Override
    public int countLikeNumberByUser(Long userId) {
        List<String> marriages = userChildInfoService.getUserMarriages(userId);
        if(marriages.isEmpty()){
            return 0;
        }
        QueryWrapper<UserLikeChildDetail> qw = new QueryWrapper<>();
        qw.lambda().in(UserLikeChildDetail::getLikeUserMarriageNumber,marriages);
        return this.count(qw);
    }

    /**
     *功能描述 检查用户是否喜欢当前用户信息
     * @author 十一
     * @date 2019/11/28 11:56
     */
    @Override
    public BaseLikeEnum checkLike(Long id, String marriageNumber) {
        if(id == null){
            return BaseLikeEnum.NotLike;
        }
        return userLikeChildDetailMapper.checkLike(id, marriageNumber);
    }

    /**
     *功能描述 获取喜欢我的子女的用户列表
     * @author 十一
     * @date 2019/11/21 10:46
     */
    @Override
    public PageInfo<UserLikeChildVO> getLikeMyChildList(User user, Integer currentPage) {
        checkVIP(user);
        currentPage = PageUtil.verifyCurrentPage(currentPage);
        PageHelper.startPage(currentPage, PageConst.PAGE_COMMON_SIZE);
        List<UserLikeChildVO> list = userLikeChildDetailMapper.getLikeMyChildList(user.getId());
        return new PageInfo<>(list);
    }

    /**
     *功能描述 检查是否是vip
     * @author 十一
     * @date 2019/11/28 15:08
     */
    public void checkVIP(User user){
        if(!user.getVip().equals(VIPEnum.NormalVip)){
            throw new AuthorityException();
        }
    }

    @Override
    public PageInfo<UserLikeChildVO> getLikedChildList(User user, Integer currentPage) {
        currentPage = PageUtil.verifyCurrentPage(currentPage);
        PageHelper.startPage(currentPage, PageConst.PAGE_COMMON_SIZE);
        List<UserLikeChildVO> list = userLikeChildDetailMapper.getLikeChildList(user.getId());
        return new PageInfo<>(list);
    }

    /**
     *功能描述 查询喜欢我的子女的缩略图
     * @author 十一
     * @date 2019/11/20 17:59
     */
    @Override
    public List<String> getLikeMyChildListByThumbnail(Long userId) {
        //查询出我的子女的姻缘号，可能有多个
        List<String> marriages = userChildInfoService.getChildMarriages(userId);
        if(marriages.isEmpty()){
            return Collections.emptyList();
        }
        QueryWrapper<UserLikeChildDetail> qw = new QueryWrapper<>();
        qw.lambda().select(UserLikeChildDetail::getUserId)
                    .in(UserLikeChildDetail::getLikeUserMarriageNumber, marriages)
                    .groupBy(UserLikeChildDetail::getUserId)
                    .orderByDesc(UserLikeChildDetail::getCreateTime);
        IPage<UserLikeChildDetail> page = new Page<>(1,3);
        IPage<UserLikeChildDetail> ipage = userLikeChildDetailMapper.selectPage(page, qw);
        List<UserLikeChildDetail> list = ipage.getRecords();
        List<Long> userIds = list.stream().map(UserLikeChildDetail::getUserId).collect(Collectors.toList());
        List<String> headImgs = userService.getUserHeadImgsByIds(userIds);
        return headImgs;
    }


    /**
     *功能描述 喜欢当前子女
     * @author 十一
     * @date 2019/11/20 17:04
     */
    @Override
    public void likeCurrentChild(User user, String marrigeNumber) {
        UserLikeChildDetail detail = new UserLikeChildDetail();
        detail.setCreateTime(new Date());
        detail.setId(IdGenerateUtil.generateId());
        detail.setLikeUserMarriageNumber(marrigeNumber);
        detail.setUserId(user.getId());
        this.save(detail);
    }

    /**
     *功能描述 统计子女被喜欢的次数
     * @author 十一
     * @date 2019/11/20 15:41
     */
    @Override
    public int countLikeNumber(String marriageNumber) {
        QueryWrapper<UserLikeChildDetail> qw = new QueryWrapper<>();
        qw.lambda().eq(UserLikeChildDetail::getLikeUserMarriageNumber,marriageNumber);
        return this.count(qw);
    }


}
