package com.sp.blinddate.module.order.param;

import com.sp.exception.ParamException;
import com.sp.utils.StringUtil;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * @Descrption 直播平台下单参数
 * @Author: 十一
 * @Date: 2019/11/16 12:05
 **/
@Data
public class LiveOrderParam implements Serializable {

    /**
     * 送礼人ID
    **/
    @NotNull(message = "送礼人id为空")
    private Long uid;

    /**
     * 收礼对象人ID（数组）:
     **/
    @NotEmpty(message = "收礼人为空")
    private List<Long> targetUid = Collections.emptyList();

    /**
     * 用户所在房间UID:
     **/
    @NotNull(message = "房间号为空")
    private String roomUid;

    /**
     * 赠送的礼物ID:
     **/
    @NotNull(message = "礼物id为空")
    private String giftId;

    /**
     * 赠送的礼物数量:
     **/
    @NotNull(message = "礼物数量为空")
    private Integer giftNum;
    /**
     * 赠送的礼物名称:
     **/
    @NotNull(message = "礼物名称为空")
    private String giftName;

    /**
     * 送礼人扣减的总
     **/
    @NotNull(message = "豆子为空")
    private Integer totalFee;
    /**
     * 接收人的每人收成
     **/
    @NotNull(message = "收款人收入")
    private Integer perIncome;

    public void setTargetUid(String targetUid) {
        if(StringUtil.isBlank(targetUid)){
            throw new ParamException("收礼对象ID为空");
        }
        this.targetUid = new ArrayList<>();
        List<String> list = Arrays.asList(targetUid.split(","));
        for (String s : list) {
            this.targetUid.add(Long.valueOf(s));
        }
    }
}
