package com.sp.blinddate.module.order.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.sp.blinddate.module.order.entity.OrderDetail;
import com.sp.blinddate.module.order.entity.TradeProcessError;
import com.sp.blinddate.module.order.service.IOrderDetailService;
import com.sp.blinddate.module.order.service.IOrderProcessScheduleService;
import com.sp.blinddate.module.order.service.ITradeProcess;
import com.sp.blinddate.module.order.service.ITradeProcessErrorService;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.utils.IdGenerateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @Author: 十一
 * @Date: 2019/9/28 11:57
 * @Descrption 付款成功后做具体的业务处理
 **/
@Service
@Transactional
public class OrderProcessScheduleServiceImpl implements IOrderProcessScheduleService {

    private static final Logger LOG = LoggerFactory.getLogger(OrderProcessScheduleServiceImpl.class);

    @Autowired
    private IOrderDetailService orderDetailService;

    @Autowired
    private IUserService userService;

    @Autowired
    private VipTradeProcessImpl vipTradeProcess;

    @Autowired
    private ITradeProcessErrorService tradeProcessErrorService;

    /**
     * 功能描述 付费成功后具体业务处理的调度方法
     *
     * @author 十一
     * @date 2019/9/28 15:42
     */
    @Override
    public void scheduleOrderProcess(String token, String orderNo) {
        QueryWrapper<OrderDetail> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(OrderDetail::getOrderNo, orderNo);
        OrderDetail orderDetail = orderDetailService.getOne(wrapper);
        String orderType = orderDetail.getBusiOrderType();
        ITradeProcess tradeProcess = getTradeProcess(orderType);
        //查询这个用户的信息
        User user = userService.getById(orderDetail.getUserId());
        //支付模块应该设计一个系统的余额，记录系统的余额动态
        try {
            tradeProcess.process(token,user, orderDetail);
            //如果执行错误，将当前订单详情记录到异常订单表中
        } catch (Exception e) {
            TradeProcessError tradeErrorInfo = new TradeProcessError();
            tradeErrorInfo.setCreateTime(new Date());
            tradeErrorInfo.setErrorMessage(e.getMessage());
            tradeErrorInfo.setOrderNo(orderNo);
            tradeErrorInfo.setId(IdGenerateUtil.generateId());
            tradeErrorInfo.setStatus(TradeProcessError.STATUS_UNPROCESS);
            tradeProcessErrorService.save(tradeErrorInfo);
            LOG.error("{}类型订单{}执行业务出错", orderDetail, orderDetail);
        }
    }

    public ITradeProcess getTradeProcess(String orderType) {
        ITradeProcess itp;
        switch (orderType) {
            default:
                itp = vipTradeProcess;
                break;
        }
        return itp;
    }
}
