package com.sp.blinddate.module.order.service.impl;

import com.sp.blinddate.module.order.entity.Order;
import com.sp.blinddate.module.order.ienum.OrderStatusEnum;
import com.sp.blinddate.module.order.ienum.PayStatusEnum;
import com.sp.blinddate.module.order.ienum.PayTypeEnum;
import com.sp.blinddate.module.order.service.IOrderDetailService;
import com.sp.blinddate.module.order.service.IOrderProcessScheduleService;
import com.sp.blinddate.module.order.service.IOrderProcessService;
import com.sp.blinddate.module.order.service.IOrderService;
import com.sp.exception.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @Author: 十一
 * @Date: 2019/9/26 19:28
 * @Descrption
 **/
@Service
@Transactional
public class OrderProcessServiceImpl implements IOrderProcessService {

    private static final Logger LOG = LoggerFactory.getLogger(OrderProcessServiceImpl.class);

    @Autowired
    private IOrderProcessScheduleService orderProcessScheduleService;

    @Autowired
    private IOrderService orderService;

    @Autowired
    private IOrderDetailService orderDetailService;

    /**
     * 订单支付完成
     * 注意：
     * 1、接口幂等，只有支付成功才会调用这个接口
     * 2、不要开启事务。跨系统调用，如果使用事务，会导致不在一个事务中，业务系统不能感知订单状态的变化
     *
     * @param token
     * @param orderNo
     * @param payType
     * @param payNo
     * @param payAppId
     * @return
     */
    @Override
    public void processTask(String token, String orderNo, PayTypeEnum payType, String payNo, String payAppId, String paySellerId) {
        Order orderPO = orderService.getById(orderNo);
        if(orderPO == null) {
            throw new ServiceException("订单不存在");
        }
        if(orderPO.getPayStatus().equals(PayStatusEnum.Payed)) {
            if(orderPO.getPayType().equals(payType)
                    && orderPO.getPayNo().equals(payNo)) {
                LOG.error("支付结果重复回调，重复消费！订单号为：{}",orderPO.getOrderNo());
                return;
            }
        }
        if(orderPO.getOrderStatus() != OrderStatusEnum.Todo) {
            throw new ServiceException("订单状态异常");
        }
        boolean r = orderService.updateOrderPayed(orderNo, payType, payNo, payAppId, paySellerId);
        boolean detailUpdateResult = orderDetailService.updatePayStatus(orderNo,PayStatusEnum.Payed);
        if(!r || !detailUpdateResult) {
            LOG.warn("订单：{}支付成功，修改订单状态失败",orderNo);
            throw new ServiceException("支付成功，修改订单状态失败");
        }
        //处理具体的业务
        orderProcessScheduleService.scheduleOrderProcess(token,orderNo);
    }
}
