package com.sp.blinddate.module.order.service.impl;

import com.sp.blinddate.module.order.entity.Order;
import com.sp.blinddate.module.order.ienum.PayTypeEnum;
import com.sp.blinddate.module.order.service.IOrderProcessService;
import com.sp.blinddate.module.order.service.IOrderResultWXPayService;
import com.sp.blinddate.module.order.service.IOrderService;
import com.sp.exception.ServiceException;
import com.sp.plugin.pay.AbstractWXPayService;
import com.sp.plugin.pay.wxpay.dto.WXPayNotifyUrlDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Map;

/**
 * @Author: 十一
 * @Date: 2019/9/26 19:19
 * @Descrption
 **/
@Service
@Transactional
public class OrderResultWXPayServiceImpl implements IOrderResultWXPayService {

    @Autowired
    private IOrderService orderService;

    @Autowired
    private IOrderProcessService orderProcessService;

    @Autowired
    private AbstractWXPayService abstractWXPayService;


    @Override
    public void notifyUrl(String token, PayTypeEnum payType, Map<String, String> params) {
        if(payType==null) {
            throw new ServiceException("附加数据缺少参数");
        }
        //商户系统对于支付结果通知的内容一定要做签名验证,
        //并校验返回的订单金额是否与商户侧的订单金额一致，
        //防止数据泄漏导致出现“假通知”，造成资金损失。
        WXPayNotifyUrlDTO notifyUrlDTO = abstractWXPayService.checkAndGetNotifyData(null,params);
        String transactionId = notifyUrlDTO.getTransactionId();
        String outTradeNo = notifyUrlDTO.getOutTradeNo();
        BigDecimal totalAmount = notifyUrlDTO.getTotalAmount();
        Order orderPO = orderService.getById(outTradeNo);
        if(orderPO == null) {
            throw new ServiceException("订单不存在");
        }
        //将金额转换成元
        int orderTotalAmout = orderPO.getTotalAmount().multiply(BigDecimal.TEN).multiply(BigDecimal.TEN).intValue();
        if(orderTotalAmout != totalAmount.intValue()) {
            throw new ServiceException("订单金额不一致");
        }
        orderProcessService.processTask(token,outTradeNo, payType, transactionId,
                notifyUrlDTO.getAppid(), notifyUrlDTO.getMchId());
    }
}
