package com.sp.blinddate.module.order.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.blinddate.module.order.entity.OrderDetail;
import com.sp.blinddate.module.order.entity.Order;
import com.sp.blinddate.module.order.ienum.OrderStatusEnum;
import com.sp.blinddate.module.order.ienum.PayStatusEnum;
import com.sp.blinddate.module.order.ienum.PayTypeEnum;
import com.sp.blinddate.module.order.mapper.OrderMapper;
import com.sp.blinddate.module.order.ordertype.VipOrderType;
import com.sp.blinddate.module.order.param.OrderParam;
import com.sp.blinddate.module.order.service.IOrderDetailService;
import com.sp.blinddate.module.order.service.IOrderService;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.utils.EnvironmentUtil;
import com.sp.blinddate.utils.TokenUtil;
import com.sp.exception.ServiceException;
import com.sp.plugin.pay.AbstractAlipayService;
import com.sp.plugin.pay.AbstractWXPayService;
import com.sp.plugin.pay.wxpay.param.WXPayAppPayParam;
import com.sp.utils.IdGenerateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author 十一
 * @since 2019-09-24
 */
@Service
@Transactional
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

    private static final Logger LOG = LoggerFactory.getLogger(OrderServiceImpl.class);


    @Autowired
    private IOrderDetailService orderDetailService;

    @Autowired
    private AbstractAlipayService abstractAlipayService;

    @Autowired
    private AbstractWXPayService abstractWXPayService;

    /**
     *功能描述 查询订单信息
     * @author 十一
     * @date 2019/9/26 10:18
     */
    @Override
    public Order queryOrder(String orderNo) {
        Order order = this.getById(orderNo);
        return order;
    }

    /**
     *功能描述 统一的支付方法
     * @author 十一
     * @date 2019/9/24 20:00
     */
    @Override
    public Object unifiedPay(String openId, Order orderPO, PayTypeEnum payType, String clientIp) {
        String busiCode = EnvironmentUtil.getProperty("friendship.busiCode");
        //微信APP支付
        if(payType == PayTypeEnum.WXPayProgram) {
            WXPayAppPayParam appPayParam = new WXPayAppPayParam();
            appPayParam.setOutTradeNo(orderPO.getOrderNo());
            appPayParam.setTotalfee(orderPO.getTotalAmount().multiply(new BigDecimal(100)).intValue());
            appPayParam.setBody(orderPO.getSubjects());
            appPayParam.setSpbillClientIp(clientIp);
            Map<String, String> attach = new HashMap<>();
            attach.put("payType", String.valueOf(payType.getNum()));
            attach.put("token", TokenUtil.getToken());
            appPayParam.setAttach(attach);
            appPayParam.setTradeType("JSAPI");
            appPayParam.setOpenId(openId);
            return abstractWXPayService.appUnified(busiCode, appPayParam);
        }
        throw new ServiceException("不支持支付类型："+payType.getName());
    }

    @Override
    public Order createOrder(User user, OrderParam orderParam, String clientIp) {
        Order order = new Order();
        String busiCode = EnvironmentUtil.getProperty("friendship.busiCode");
        VipOrderType vipOrderType = new VipOrderType();
        order.setBusiOrderType(vipOrderType.getOrderTypeName());
        order.setBusiOrderNo(vipOrderType.getOrderTypeNo());
        //设置业务代码，用于区分产品线
        order.setBusiCode(busiCode);
        //设置订单号
        order.setOrderNo(new IdGenerateUtil().nextId());
        //设置订单状态
        order.setOrderStatus(OrderStatusEnum.Todo);
        //设置订单支付状态
        order.setPayStatus(PayStatusEnum.Topay);
        //设置支付总金额
        order.setTotalAmount(orderParam.getCurrentTotalAmount());
        //设置支付标题
        order.setSubjects("子女帮帮会员");
        //设置支付类型
        order.setPayType(orderParam.getPayType());
        //设置支付的appid
        String wxAppId = EnvironmentUtil.getProperty("wechat.appId");
        order.setPayAppId(wxAppId);
        //设置创建时间
        order.setCreateTime(new Date());
        //设置支付渠道
        order.setChannel(orderParam.getChannel());
        this.save(order);
        LOG.warn("初始化订单信息：订单号为：{}",order.getOrderNo());

        //保存订单详情
        OrderDetail orderDetail = new OrderDetail();
        orderDetail.setBusiOrderType(orderParam.getBusiOrderType());
        orderDetail.setCreateTime(new Date());
        orderDetail.setOrderPayStatus(PayStatusEnum.Topay);
        orderDetail.setOrderNo(order.getOrderNo());
        orderDetail.setOriginalPrice(orderParam.getOriginalTotalAmount());
        orderDetail.setCurrentPrice(orderParam.getCurrentTotalAmount());
        orderDetail.setQuantity(orderParam.getQuantity());
        orderDetail.setUserId(user.getId());
        orderDetail.setId(IdGenerateUtil.generateId());
        orderDetailService.save(orderDetail);
        return order;
    }


    /**
     *功能描述 更新订单状态
     * @author 十一
     * @date 2019/9/26 19:04
     */
    @Override
    public boolean updateOrderPayed(String orderNo, PayTypeEnum payType, String payNo, String payAppId, String paySellerId) {
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(Order::getPayNo, payNo)
                .set(Order::getPayAppId, payAppId)
                .set(Order::getPaySellerId, paySellerId)
                .set(Order::getPayType, payType.getNum())
                .set(Order::getOrderStatus, OrderStatusEnum.Notify.getNum())
                .set(Order::getPayStatus, PayStatusEnum.Payed.getNum())
                .eq(Order::getOrderNo, orderNo)
                .eq(Order::getPayStatus,PayStatusEnum.Topay)
                .eq(Order::getOrderStatus, OrderStatusEnum.Todo);
        boolean result = this.update(updateWrapper);
        LOG.info("订单：{}完成支付",orderNo);
        return result;
    }

    /**
     * 订单支付完成
     * 注意：
     * 1、接口幂等，只有支付成功才会调用这个接口
     * 2、不要开启事务。跨系统调用，如果使用事务，会导致不在一个事务中，业务系统不能感知订单状态的变化
     * @param orderNo
     * @param payType
     * @param payNo
     * @param payAppId
     * @param paySellerId
     * @return
     */
    public void payed(String orderNo,PayTypeEnum payType, String payNo, String payAppId, String paySellerId) {
        Order orderPO = this.getById(orderNo);
        if(orderPO == null) {
            throw new ServiceException("订单不存在");
        }
        if(orderPO.getPayStatus() == PayStatusEnum.Payed) {
            if(orderPO.getPayType()!=payType
                    || !orderPO.getPayNo().equals(payNo)) {
                throw new ServiceException("订单重复支付");
            }
            return;
        }
        if(orderPO.getOrderStatus() != OrderStatusEnum.Todo) {
            throw new ServiceException("订单状态异常");
        }
        boolean r = this.updateOrderPayed(orderNo, payType, payNo, payAppId, paySellerId);
        if(!r) {
            throw new ServiceException("更新订单状态失败");
        }
        //执行相关的业务操作
    }
}
