package com.sp.blinddate.module.order.service.impl;

import com.sp.RedisServer;
import com.sp.blinddate.base.status.VIPEnum;
import com.sp.blinddate.module.order.entity.OrderDetail;
import com.sp.blinddate.module.order.service.ITradeProcess;
import com.sp.blinddate.module.statistics.entity.ProfileStatistics;
import com.sp.blinddate.module.statistics.prefix.StatisticKeysPrefix;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.ienum.VipExpireStatusEnum;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.blinddate.utils.TokenUtil;
import com.sp.utils.CalendarUtil;
import com.sp.utils.DateUtil;
import com.sp.utils.IdGenerateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;

/**
 * @Author: 十一
 * @Date: 2019/9/28 15:48
 * @Descrption vip交易的业务处理类
 **/
@Service
@Transactional
public class VipTradeProcessImpl implements ITradeProcess {

    private static final Logger LOG = LoggerFactory.getLogger(VipTradeProcessImpl.class);

    @Autowired
    private IUserService userService;
    @Autowired
    private RedisServer redisServer;

    @Override
    public void process(String token, User user, OrderDetail orderDetail) {
        //判断是否是vip，如果是vip，获取他的过期时间，然后根据支付的情况自动为其生成vip的过期时间
        Date vipExpiredTime = user.getVipExpireDate();
        Date now = new Date();
        //如果之前不是vip
        Integer quantity = orderDetail.getQuantity();
        Date expiredTime = null;
        if (vipExpiredTime == null || vipExpiredTime.getTime() < now.getTime()) {
            //根据充值的周期设置过期时间
            expiredTime = CalendarUtil.addMonth(now, quantity);
        } else {
            expiredTime = CalendarUtil.addMonth(user.getVipExpireDate(), quantity);
        }
        user.setVipExpireDate(expiredTime);
        user.setVipExpireStatus(VipExpireStatusEnum.NotExpire);
        //设置vip会员的标示
        user.setVip(VIPEnum.NormalVip);
        //将缓存中的vip内容设置为true
        String localTime = DateUtil.dateFormat(expiredTime);
        //设置认证费用
        Date today = new Date();
        String todayStr = DateUtil.dateStrDay(today);
        ProfileStatistics ps = redisServer.get(StatisticKeysPrefix.profileStatistics, todayStr, ProfileStatistics.class);
        if(ps == null){
            ps = new ProfileStatistics();
            ps.setDate(new Date());
            ps.setId(IdGenerateUtil.generateId());
            ps.setTotalCertificationFee(BigDecimal.ZERO);
            ps.setTodayCertificationFee(BigDecimal.ZERO);
            ps.setYestodayCertificationFee(BigDecimal.ZERO);
            ps.setPublishTotal(0);
            ps.setTodayPublishTotal(0);
            ps.setYestodayPublishTotal(0);
        }
        ps.setTodayCertificationFee(ps.getTotalCertificationFee().add(orderDetail.getCurrentPrice()));
        ps.setTotalCertificationFee(ps.getTotalCertificationFee().add(orderDetail.getCurrentPrice()));
        redisServer.set(StatisticKeysPrefix.profileStatistics, todayStr,ps);
        redisServer.set(UserKeyPrefix.userInfoLoginToken, token, user);
        userService.updateById(user);
        LOG.warn("用户:{}开通了{}个月的VIP权限，过期时间至{}", user.getId(), quantity, localTime);
    }
}
