package com.sp.blinddate.module.recommand.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.sp.RedisServer;
import com.sp.blinddate.base.status.VIPEnum;
import com.sp.blinddate.base.vo.AddressComponent;
import com.sp.blinddate.common.BaseLikeEnum;
import com.sp.blinddate.module.chlidinfo.keyprefix.UserInfoKeyPrefix;
import com.sp.blinddate.module.geo.keyprefix.GeoKeyPrefix;
import com.sp.blinddate.module.likechildinfo.service.IUserLikeChildDetailService;
import com.sp.blinddate.module.recommand.param.RecommandParam;
import com.sp.blinddate.module.recommand.service.RecommandService;
import com.sp.blinddate.module.recommand.vo.RecommandVO;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.entity.UserChildInfo;
import com.sp.blinddate.module.user.ienum.GenderEnum;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.module.user.service.IUserChildInfoService;
import com.sp.blinddate.utils.EnvironmentUtil;
import com.sp.blinddate.utils.ParseAliMapUtil;
import com.sp.utils.CalendarUtil;
import com.sp.utils.DateUtil;
import com.sp.utils.OkHttpUtil;
import org.redisson.api.RBloomFilter;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @Descrption
 * @Author: 十一
 * @Date: 2019/11/21 15:28
 **/
@Service
public class RecommandServiceImpl implements RecommandService {

    /**
     * 推荐数量为6
    **/
    private Integer commandPageSize = 6;

    /**
     * 每个性别类型的数量是3个
    **/
    private Integer commandGenderPageSize = 3;


    @Autowired
    private RedisServer redisServer;

    @Autowired
    private IUserLikeChildDetailService userLikeChildDetailService;

    @Autowired
    private RedissonClient client;

    @Autowired
    private IUserChildInfoService userChildInfoService;


    /**
     *功能描述 推荐用户信息
     * @author 十一
     * @date 2019/11/28 15:29
     */
    @Override
    public List<RecommandVO> recommand(User user, RecommandParam param) {
        List<Long> resultList = new ArrayList<>();
        String todayStr = DateUtil.dateStrDay(new Date());
        //判断是否是游客
        if (user == null || user.getVip().equals(VIPEnum.Normal)) {
            //判断是否已经推荐过
            boolean exists = redisServer.exists(UserKeyPrefix.userInfoRecommandFlag,
                    todayStr+":"+user.getId()+":"+param.getGender().getNum());
            if (exists) {
                List list = redisServer.get(UserKeyPrefix.userRecommandHistory,
                        todayStr+":"+user.getId()+":"+param.getGender().getNum(), List.class);
                if(!list.isEmpty()){
                    return list;
                }
            }
        }
        //标示用户已经推荐过
        redisServer.set(UserKeyPrefix.userInfoRecommandFlag,
                todayStr+":"+user.getId()+":"+param.getGender().getNum(), true);
        Long filterExpireTime = Long.parseLong(EnvironmentUtil.getProperty("user.bloomFilter.expireTime"));
        redisServer.resetExpireSeconds(UserKeyPrefix.userInfoRecommandFlag,
                todayStr+":"+user.getId()+":"+param.getGender().getNum(),filterExpireTime.intValue());
        AddressComponent vo = redisServer.get(UserKeyPrefix.userLocationCity, user.getId()+"", AddressComponent.class);
        if (vo == null) {
            Map<String, String> map = new HashMap<>();
            map.put("location", param.getCoordinate());
            map.put("radius", "1000");
            map.put("extensions", "base");
            map.put("roadlevel", "0");
            map.put("batch", "false");
            map.put("key", EnvironmentUtil.getProperty("alimap.key"));
            //获取用户的地址
            JSONObject obj = OkHttpUtil.getRequest(EnvironmentUtil.getProperty("alimap.url"), map);
            vo = ParseAliMapUtil.parse(obj);
            redisServer.set(UserKeyPrefix.userLocationCity, user.getId()+"", vo);
        }
        //用户当前所在的省
        String province = vo.getProvince();
        UserChildInfo info = null;
        List<UserChildInfo> childInfos = userChildInfoService.getByUserId(user.getId());
        if(!childInfos.isEmpty()){
            info = childInfos.get(0);
        }else{
            info = new UserChildInfo();
            info.setResidenceCity(vo.getCity());
            info.setResidenceProvince(vo.getProvince());
            info.setCensusProvince(vo.getProvince());
            info.setCensusCity(vo.getCity());
            info.setCensusCity(vo.getCity());
        }
        //TODO 未判断用户多个子女的情况
        //用户的户籍地
        String censusProvince = info.getCensusProvince();
        RBloomFilter<Object> filter = client.getBloomFilter(UserInfoKeyPrefix.USER_RECOMMAND_FILTER+":"+ user.getId());
        Long expireTime = redisServer.ttlWithFilter(UserInfoKeyPrefix.USER_RECOMMAND_FILTER +":"+ user.getId());
        if (expireTime == -2){
            filter.tryInit(500000L, 0.03);
//            redisServer.expireFilter(UserInfoKeyPrefix.USER_RECOMMAND_FILTER +":"+ user.getId(), -1);
            //删除标示
            redisServer.delete(UserKeyPrefix.userRecommandLoopStatus, user.getId() + "");
        }
        //判断性别
        GenderEnum requestGender = param.getGender();
        //判断用户是否在户籍所在地,如果是则指需要推荐用户户籍地所在的地方的信息
        if (province.equals(censusProvince)) {
            //如果性别不是其他，那么四个异性
            if (!requestGender.equals(GenderEnum.Other)) {
                resultList = recommandResidenceAroundNerberProvince(vo, param, info, filter);
            } else {
                //查询两个男性，两个女性
                resultList = recommandResidenceAroundNerberProvinceForThreeMaleAndThreeFemale(vo, param, info, filter);
            }
        } else {
            //如果性别不是是其他，那么四个异性
            if (!requestGender.equals(GenderEnum.Other)){
                //从用户所在地推荐数据
                resultList = recommandResidence(vo, param, info, filter);
                ok:
                if (resultList.size() < commandPageSize) {
                    //推荐户籍地所有市的数据,有可能数据没有四条
                    String provinceKey = info.getCensusProvince() + ":*" + GenderEnum.getReverseGender(requestGender).getNum();
                    List<String> provinceCities = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
                    for (String inCity : provinceCities) {
                        Set<Object> inSet = redisServer.zReverseRange(inCity, 0, -1);
                        Iterator<Object> itera = inSet.iterator();
                        while (itera.hasNext()) {
                            Object next = itera.next();
                            boolean contains = filter.contains(next.toString());
                            if (!contains) {
                                filter.add(next.toString());
                                resultList.add(Long.valueOf(next.toString()));
                                if (resultList.size() >= commandPageSize) {
                                    break ok;
                                }
                            }

                        }
                    }
                    //如果没有匹配到，那么从所在地附近匹配
                    if(resultList.size() < commandPageSize){
                        List<Long> result = recommandResidenceAroundNerberProvince(vo, param, info, filter);
                        resultList.addAll(result);
                    }
                }
            } else {
                //从用户所在地推荐，用户所在地可能与用户的定位位置不一致
                resultList = recommandResidenceForGenderOther(vo, param, info, filter);
                if (resultList.size() < commandPageSize) {
                    Integer femaleCount = redisServer.get(UserKeyPrefix.userRecommandFemaleCount, user.getId() + "", Integer.class);
                    int needFemaleCount = commandGenderPageSize;
                    if(femaleCount != null){
                        needFemaleCount = commandGenderPageSize - femaleCount;
                    }
                    Integer maleCount = redisServer.get(UserKeyPrefix.userRecommandMaleCount, user.getId() + "", Integer.class);
                    int needMaleCount = commandGenderPageSize;
                    if(maleCount != null){
                        needMaleCount = commandGenderPageSize - maleCount;
                    }
                    //判断女性用户信息数据还需要几条
                    if(needFemaleCount > 0){
                        List<Long> list = this.recommandCensusWhitGenderAndCount(GenderEnum.Female,needFemaleCount, info, filter);
                        if(!list.isEmpty()){
                            needFemaleCount -= list.size();
                            resultList.addAll(list);
                        }
                    }
                    if(needMaleCount > 0){
                        List<Long> list = this.recommandCensusWhitGenderAndCount(GenderEnum.Male,needMaleCount, info, filter);
                        if(!list.isEmpty()){
                            needFemaleCount -= list.size();
                            resultList.addAll(list);
                        }
                    }
                    //如果没有匹配到，那么从所在地附近匹配
                    if(needFemaleCount > 0){
                        List<Long> result = recommandResidenceAroundNerberProvinceByGenderWithCount(needFemaleCount,GenderEnum.Female,vo, param, info, filter);
                        needFemaleCount -= result.size();
                        resultList.addAll(result);
                    }
                    if(needMaleCount > 0){
                        List<Long> result = recommandResidenceAroundNerberProvinceByGenderWithCount(needMaleCount,GenderEnum.Male,vo, param, info, filter);
                        needMaleCount -= result.size();
                        resultList.addAll(result);
                    }
                    //如果还是没有四条数据，判断是男性用户数据少还是女性少，如果都少则不处理,否则从异性用户池中查找
                    if(needFemaleCount > 0 && needFemaleCount > 0){
                        //不做任何操作
                    }else if(needFemaleCount > 0){
                        List<Long> result = recommandResidenceAroundNerberProvinceByGenderWithCount(needFemaleCount,GenderEnum.Male,vo, param, info, filter);
                        resultList.addAll(result);
                    }else if (needMaleCount > 0){
                        List<Long> result = recommandResidenceAroundNerberProvinceByGenderWithCount(needMaleCount,GenderEnum.Female,vo, param, info, filter);
                        resultList.addAll(result);
                    }

                }
            }
        }
        List<RecommandVO> recommandList = getRecommandListByInfoIds(user, resultList);
        List<RecommandVO> historyList = new ArrayList<>();
        if(recommandList.size() < commandPageSize){
            //保存上次推荐的六条数据
            List<RecommandVO> lastRecommand = redisServer.get(UserKeyPrefix.userRecommandHistory,
                    todayStr+":"+user.getId()+":"+param.getGender().getNum(), List.class);
            if(lastRecommand == null || lastRecommand.isEmpty()){
                redisServer.set(UserKeyPrefix.userRecommandHistory,
                        todayStr+":"+user.getId()+":"+param.getGender().getNum(), recommandList);
            }else{
                historyList.addAll(lastRecommand);
                historyList.addAll(recommandList);
                while(historyList.size()> commandPageSize){
                    historyList.remove(0);
                }
                redisServer.set(UserKeyPrefix.userRecommandHistory,
                        todayStr+":"+user.getId()+":"+param.getGender().getNum(), historyList);
            }
        }else{
            if(recommandList.size() > commandPageSize){
                System.err.println("推荐的数量出错:"+recommandList.size());
            }
            redisServer.set(UserKeyPrefix.userRecommandHistory,
                    todayStr+":"+user.getId()+":"+param.getGender().getNum(), recommandList);
        }
        Boolean recommandLoop = redisServer.exists(UserKeyPrefix.userRecommandLoopStatus, user.getId() + "");
        if(recommandList.isEmpty() && !recommandLoop){
            //标示用户已经浏览完，然后删除所有的过滤条件，重新推荐
            redisServer.set(UserKeyPrefix.userRecommandLoopStatus, user.getId() + "", 1);
            //删除过滤器
            redisServer.deleteFilter(UserInfoKeyPrefix.USER_RECOMMAND_FILTER + ":" + user.getId());
            List<RecommandVO> result = recommand(user, param);
            recommandList = result;
        }
        return recommandList;
    }


    /**
     *功能描述 在户籍地所以在地根据性别推荐指定数量的用户
     * @author 十一
     * @date 2019/12/2 20:33
     */
    public List<Long> recommandCensusWhitGenderAndCount(GenderEnum gender,Integer count,UserChildInfo info,RBloomFilter<Object> filter){
        List<Long> resultList = new ArrayList<>();
        //推荐户籍地所有市的数据,不判断性别 有可能数据没有四条
        String provinceKey = info.getCensusProvince() + ":*:"+gender.getNum();
        List<String> provinceCities = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
        for (String inCity : provinceCities) {
            Set<Object> inSet = redisServer.zReverseRange(inCity, 0, -1);
            Iterator<Object> itera = inSet.iterator();
            while (itera.hasNext()) {
                Object next = itera.next();
                boolean contains = filter.contains(next.toString());
                if (!contains) {
                    filter.add(next.toString());
                    resultList.add(Long.valueOf(next.toString()));
                    if (resultList.size() >= count) {
                        return resultList;
                    }
                }
            }
        }
        return resultList;
    }

    /**
     * 功能描述 根据用户id查询出推荐信息
     *
     * @author 十一
     * @date 2019/11/28 11:30
     */
    @Override
    public List<RecommandVO> getRecommandListByInfoIds(User user, List<Long> userInfos) {
        if(userInfos.isEmpty()){
            return Collections.emptyList();
        }
        List<UserChildInfo> infos = userChildInfoService.getRecommandListByInfoIds(userInfos);
        List<RecommandVO> list = new ArrayList<>();
        if (!infos.isEmpty()) {
            for (UserChildInfo info : infos) {
                RecommandVO vo = new RecommandVO();
                vo.setIncome(info.getIncome() == null ? "": info.getIncome().getName());
                vo.setHeight(info.getHeight() == null ? "": info.getHeight().getName());
                vo.setEducation(info.getEducation() == null ? "": info.getEducation().getName());
                vo.setAge(CalendarUtil.getYearsFromDate(info.getDateOfBirth()));
                vo.setBirth(info.getDateOfBirth());
                vo.setCarrer(info.getCarrer().getName());
                vo.setCoverImg(info.getCoverImg());
                vo.setCensusProvince(info.getCensusProvince());
                vo.setCensusCity(info.getCensusCity());
                vo.setMarriageNumber(info.getMarriageNumber());
                vo.setResidence(info.getResidenceCity());
                vo.setStatus(info.getStatus());
                vo.setUserInfoId(info.getId());
                vo.setGender(info.getGender().getName());
                BaseLikeEnum result = userLikeChildDetailService.checkLike(user == null? null:user.getId(),
                        info.getMarriageNumber());
                vo.setIsLiked(result);
                list.add(vo);
            }
        }
        return list;
    }


    /**
     * 功能描述 推荐用户所在地的数据
     *
     * @author 十一
     * @date 2019/11/27 21:11
     */
    public List<Long> recommandResidence(AddressComponent vo, RecommandParam param, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> resultList = new ArrayList<>();
        String city = vo.getCity();
        String province = vo.getProvince();
        GenderEnum requestGender = param.getGender();
        if (param.getGender() != GenderEnum.Other) {
            String key = province + ":" + city +":"+ GenderEnum.getReverseGender(requestGender).getNum();
            //从用户所在的省市开始匹配
            Set<Object> set = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, key, 0, -1);
            Iterator<Object> iter = set.iterator();
            while (iter.hasNext()) {
                Object next = iter.next();
                boolean contains = filter.contains(next.toString());
                if (!contains) {
                    filter.add(next.toString());
                    resultList.add(Long.valueOf(next.toString()));
                    if (resultList.size() >= commandPageSize) {
                        break;
                    }
                }
            }
            ok:
            //判断推荐列表中是否已经有四条数据
            if (resultList.size() < commandPageSize) {
                //推荐同省的其他市的数据
                String provinceKey = info.getResidenceProvince() + ":*" + GenderEnum.getReverseGender(requestGender).getNum();
                List<String> provinceCities = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
                for (String inCity : provinceCities) {
                    if (!inCity.equals(city)) {
                        Set<Object> inSet = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, inCity, 0, -1);
                        Iterator<Object> itera = inSet.iterator();
                        while (itera.hasNext()) {
                            Object next = iter.next();
                            boolean contains = filter.contains(next.toString());
                            if(!contains){
                                filter.add(next.toString());
                                resultList.add(Long.valueOf(next.toString()));
                                if (resultList.size() >= commandPageSize) {
                                    break ok;
                                }
                            }
                        }
                    }
                }
            }
        }
        return resultList;
    }

    /**
     * 功能描述 从用户所在地推荐两名男性，两名女性
     *
     * @author 十一
     * @date 2019/11/27 21:11
     */
    public List<Long> recommandResidenceForGenderOther(AddressComponent vo, RecommandParam param, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> resultList = new ArrayList<>();
        List<Long> maleList = new ArrayList<>();
        List<Long> femaleList = new ArrayList<>();
        maleList = recommandResidenceForTwoByGender(vo, GenderEnum.Male, info, filter);
        femaleList = recommandResidenceForTwoByGender(vo, GenderEnum.Female, info, filter);
        redisServer.set(UserKeyPrefix.userRecommandMaleCount, info.getUserId()+"", maleList.size());
        redisServer.set(UserKeyPrefix.userRecommandFemaleCount, info.getUserId()+"", femaleList.size());
        resultList.addAll(maleList);
        resultList.addAll(femaleList);
        return resultList;
    }


    /**
     * 功能描述 根据性别在用户所在地推荐两名用户信息
     *
     * @author 十一
     * @date 2019/11/28 10:15
     */
    public List<Long> recommandResidenceForTwoByGender(AddressComponent vo, GenderEnum gender, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> resultList = new ArrayList<>();
        //用户当前所在的省
        String city = vo.getCity();
        String key = info.getResidenceProvince() + ":" + info.getResidenceCity() +":"+gender.getNum();
        //从用户所在的省市开始匹配
        Set<Object> set = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, key, 0, -1);
        Iterator<Object> iter = set.iterator();
        while (iter.hasNext()) {
            Object next = iter.next();
            boolean contains = filter.contains(next.toString());
            if (!contains) {
                filter.add(next.toString());
                resultList.add(Long.valueOf(next.toString()));
                if (resultList.size() >= commandGenderPageSize) {
                    break;
                }
            }
        }
        ok:
        //判断推荐列表中是否已经有四条数据
        if (resultList.size() < commandGenderPageSize) {
            //推荐同省的其他市的数据
            String provinceKey = info.getResidenceProvince() + ":*" + gender.getNum();
            List<String> provinceCities = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
            for (String inCity : provinceCities) {
                if (!inCity.equals(city)) {
                    Set<Object> inSet = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, inCity, 0, -1);
                    Iterator<Object> itera = inSet.iterator();
                    while (itera.hasNext()) {
                        Object next = iter.next();
                        boolean contains = filter.contains(next.toString());
                        if(!contains){
                            filter.add(next.toString());
                            resultList.add(Long.valueOf(next.toString()));
                            if (resultList.size() >= commandGenderPageSize) {
                                break ok;
                            }
                        }
                    }
                }
            }
        }
        return resultList;
    }



    /**
     * 功能描述 推荐所在地的信息
     *
     * @author 十一
     * @date 2019/11/27 21:05
     */
    public List<Long> recommandResidenceAroundNerberProvinceByGenderWithCount(Integer count,GenderEnum gender,AddressComponent vo, RecommandParam param, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> list = recommandNerberPrivinceByGenderAndCount(count,gender, vo, info, filter);
        return list;
    }


    /**
     * 功能描述 推荐所在地的信息
     *
     * @author 十一
     * @date 2019/11/27 21:05
     */
    public List<Long> recommandResidenceAroundNerberProvince(AddressComponent vo, RecommandParam param, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> resultList = new ArrayList<>();
        Integer totalCount = commandPageSize;
        if (param.getGender() != GenderEnum.Other) {
            GenderEnum gender = GenderEnum.getReverseGender(param.getGender());
            resultList = recommandNerberPrivinceByGenderAndCount(totalCount, gender,vo, info, filter);
            //先去用户所在地查找，如果用户的所在地推完了，就到用户的户籍地去查找
        }else{
            //分别按照男女的个数查找，先查找女性的个数，如果个数不对应，那么缺少的个数全部查找男性
            Integer female = redisServer.get(UserKeyPrefix.userRecommandFemaleCount, info.getUserId() + "", int.class);
            Integer male = redisServer.get(UserKeyPrefix.userRecommandMaleCount, info.getUserId() + "", int.class);
            if(female == null || female == 0){
                totalCount = commandGenderPageSize;
            }else{
                totalCount = commandGenderPageSize - female;
            }
            //查找女性资料，如果少于两个，那么将缺少的个数用于查询男性的信息
            List<Long> femaleList = recommandNerberPrivinceByGenderAndCount(totalCount, GenderEnum.Female, vo, info, filter);
            if(femaleList.isEmpty()){
                //计算出需要查询到的男性用户信息个数
                if(male == null){
                    totalCount = commandPageSize;
                }else{
                    totalCount = commandGenderPageSize - male + totalCount;
                }
            }else{
                if(male == null){
                    totalCount = commandPageSize - femaleList.size();
                }else{
                    totalCount = commandGenderPageSize - male + (commandGenderPageSize - femaleList.size());
                }
            }
            //查找女性资料，如果少于两个，那么将缺少的个数用于查询男性的信息
            List<Long> maleList = recommandNerberPrivinceByGenderAndCount(totalCount, GenderEnum.Male, vo, info, filter);
            resultList.addAll(femaleList);
            resultList.addAll(maleList);
        }
        return resultList;
    }

    /**
     *功能描述 根据性别推荐需要匹配的数量
     * @author 十一
     * @date 2019/12/2 16:57
     */
    public List<Long> recommandNerberPrivinceByGenderAndCount(Integer count,GenderEnum gender,
                          AddressComponent vo, UserChildInfo info, RBloomFilter<Object> filter){
        List<Long> resultList = new ArrayList<>();
        //用户当前所在的省
        String province = vo.getProvince();
        String city = vo.getCity();
        String key = vo.getProvince() + ":" + vo.getCity() + ":"+gender.getNum();
        //从用户所在的省市开始匹配
        Set<Object> set = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, key, 0, -1);
        Iterator<Object> iter = set.iterator();
        while (iter.hasNext()) {
            Object next = iter.next();
            boolean contains = filter.contains(next.toString());
            if (!contains) {
                filter.add(next.toString());
                resultList.add(Long.valueOf(next.toString()));
                if (resultList.size() >= count) {
                    break;
                }
            }
        }
        ok:
        //判断推荐列表中是否已经有四条数据
        if (resultList.size() < count) {
            //推荐同省的其他市的数据
            String provinceKey = vo.getProvince() + ":*" + gender.getNum();
            List<String> provinceCities = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
            for (String inCity : provinceCities) {
                if (!inCity.equals(city)) {
                    Set<Object> inSet = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, inCity, 0, -1);
                    Iterator<Object> itera = inSet.iterator();
                    while (itera.hasNext()) {
                        Object next = iter.next();
                        boolean contains = filter.contains(next.toString());
                        if(!contains){
                            resultList.add(Long.valueOf(next.toString()));
                            filter.add(next.toString());
                            if (resultList.size() >= count) {
                                break ok;
                            }
                        }
                    }

                }
            }
        }
        ok1:
        //判断推荐列表中是否已经有四条数据
        if (resultList.size() < count) {
            //推荐邻省的其他市的数据，去掉最后一个字
            String macthProvince = province.substring(0, province.length() - 1);
            Set<Object> nerboerProvinces = redisServer.zrange(GeoKeyPrefix.provinceDistance, macthProvince, 0, -1);
            Iterator<Object> iterator = nerboerProvinces.iterator();
            while (iterator.hasNext()) {
                //获取邻省的名字
                Object next = iterator.next();
                String provinceKey = next.toString() + "*" + gender.getNum();
                //根据省名查询出所有的市级列表
                List<String> nerberProvinceList = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
                for (String nerberKey : nerberProvinceList) {
                    Set<Object> nerberInfo = redisServer.zReverseRange(nerberKey, 0, -1);
                    Iterator<Object> nerberIter = nerberInfo.iterator();
                    while (nerberIter.hasNext()) {
                        Object nerberNext = nerberIter.next();
                        boolean contains = filter.contains(nerberNext.toString());
                        if(!contains){
                            filter.add(nerberNext.toString());
                            resultList.add(Long.valueOf(nerberNext.toString()));
                            if (resultList.size() >= count) {
                                break ok1;
                            }
                        }
                    }
                }

            }
        }
        return resultList;
    }

    /**
     * 功能描述 推荐所在地的信息
     *
     * @author 十一
     * @date 2019/11/27 21:05
     */
    public List<Long> recommandResidenceAroundNerberProvinceForThreeMaleAndThreeFemale(AddressComponent vo, RecommandParam param, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> resultList = new ArrayList<>();
        List<Long> maleList = new ArrayList<>();
        List<Long> femaleList = new ArrayList<>();
        //获取男性记录
        maleList = recommandThreeResultForGender(vo, GenderEnum.Male, info, filter);
        femaleList = recommandThreeResultForGender(vo, GenderEnum.Female, info, filter);
        resultList.addAll(maleList);
        resultList.addAll(femaleList);
        return resultList;
    }

    /**
     * 功能描述 在用户所在地的省份用户池子中根据用户的性别选择两条数据
     *
     * @author 十一
     * @date 2019/11/28 10:00
     */
    public List<Long> recommandThreeResultForGender(AddressComponent vo, GenderEnum gender, UserChildInfo info, RBloomFilter<Object> filter) {
        List<Long> resultList = new ArrayList<>();
        //用户当前所在的省
        String province = vo.getProvince();
        String city = vo.getCity();
        String key = info.getResidenceProvince() + ":" + info.getResidenceCity() +":"+ gender.getNum();
        //从用户所在的省市开始匹配
        Set<Object> set = redisServer.zReverseRange(UserInfoKeyPrefix.userPool, key, 0, -1);
        Iterator<Object> iter = set.iterator();
        while (iter.hasNext()) {
            Object next = iter.next();
            boolean contains = filter.contains(next.toString());
            if (!contains) {
                filter.add(next.toString());
                resultList.add(Long.valueOf(next.toString()));
                if (resultList.size() >= commandGenderPageSize) {
                    break;
                }
            }
        }
        ok:
        //判断推荐列表中是否已经有四条数据
        if (resultList.size() < commandGenderPageSize) {
            //推荐同省的其他市的数据
            String provinceKey = info.getResidenceProvince() + ":*" + gender.getNum();
            List<String> provinceCities = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
            for (String inCity : provinceCities) {
                if (!inCity.equals(city)) {
                    Set<Object> inSet = redisServer.zReverseRange(inCity, 0, -1);
                    Iterator<Object> itera = inSet.iterator();
                    while (itera.hasNext()) {
                        Object next = itera.next();
                        boolean contains = filter.contains(next.toString());
                        if(!contains){
                            filter.add(next.toString());
                            resultList.add(Long.valueOf(next.toString()));
                            if (resultList.size() >= commandGenderPageSize) {
                                break ok;
                            }
                        }
                    }

                }
            }
        }
        ok1:
        //判断推荐列表中是否已经有四条数据
        if (resultList.size() < commandGenderPageSize) {
            String provinceWhitOutSubfix = province.substring(0,province.length() -1);
            //推荐邻省的其他市的数据
            Set<Object> nerboerProvinces = redisServer.zrange(GeoKeyPrefix.provinceDistance, provinceWhitOutSubfix, 0, -1);
            Iterator<Object> iterator = nerboerProvinces.iterator();
            while (iterator.hasNext()) {
                //获取邻省的名字
                Object next = iterator.next();
                String provinceKey = next.toString() + "*" + gender.getNum();
                //根据省名查询出所有的市级列表
                List<String> nerberProvinceList = redisServer.scan(UserInfoKeyPrefix.userPool.getPrefix() + provinceKey);
                for (String nerberKey : nerberProvinceList) {
                    Set<Object> nerberInfo = redisServer.zReverseRange(nerberKey, 0, -1);
                    Iterator<Object> nerberIter = nerberInfo.iterator();
                    while (nerberIter.hasNext()) {
                        Object nerberNext = nerberIter.next();
                        boolean contains = filter.contains(nerberNext.toString());
                        if(!contains){
                            filter.add(nerberNext.toString());
                            resultList.add(Long.valueOf(nerberNext.toString()));
                            if (resultList.size() >= commandGenderPageSize) {
                                break ok1;
                            }
                        }

                    }
                }
            }
        }
        return resultList;
    }
}
