package com.sp.blinddate.module.user.param;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.sp.blinddate.module.user.ienum.*;
import com.sp.blinddate.validator.SensitiveWords;
import com.sp.constant.P;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * @Descrption
 * @Author: 十一
 * @Date: 2019/11/18 18:13
 **/
@Data
@ApiModel(value = "用户信息保存对象参数")
public class UserFullInfoParam implements Serializable {

    @ApiModelProperty(name="userInfoId",value="用户信息id")
    private Long userInfoId;

    @NotNull(message = "图片列表为空")
    @ApiModelProperty(name="images",value="图片列表")
    private List<String> images;

    @NotNull(message = "封面为空")
    @ApiModelProperty(name="coverImg",value="封面图片")
    private String coverImg;
    
    /*********************  用户子女基本信息  ***********************/

    @NotNull(message = "性别为空")
    @ApiModelProperty(name="gender",value="性别1:男,2:女")
    private GenderEnum gender;

    @NotNull(message = "出生日期为空")
    @JsonFormat(pattern = "yyyy-MM-dd",timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ApiModelProperty(name="dateOfBirth",value="出生日期")
    private Date dateOfBirth;

    @NotNull(message = "居住地为空(省）")
    @ApiModelProperty(name = "residenceProvince",value = "居住地(省）")
    private String residenceProvince;

    @NotNull(message = "居住地为空(市)")
    @ApiModelProperty(name = "residenceCity",value = "居住地(市）")
    private String residenceCity;

    @NotNull(message = "户籍为空")
    @ApiModelProperty(name="censusProvince",value="户籍(省）")
    private String censusProvince;

    @NotNull(message = "户籍为空")
    @ApiModelProperty(name="censusCity",value="户籍(市）")
    private String censusCity;

    @NotNull(message = "学历为空")
    @ApiModelProperty(name="education",value="学历")
    private EducationEnum education;

    @NotNull(message = "身高为空")
    @ApiModelProperty(name="height",value="身高")
    private UserHegihtEnum height;

    @NotNull(message = "职业为空")
    @ApiModelProperty(name="carrer",value="职业")
    private CarrerEnum carrer;

    @NotNull(message = "手机号码为空")
    @Pattern(regexp = P.PHONE_PATTERN, message = "手机号码格式不正确")
    @ApiModelProperty(name="phone",value="手机号码")
    private String phone;

    @NotNull(message = "微信号为空")
    @SensitiveWords(message = "微信号包含敏感词")
    @ApiModelProperty(name="wechat",value="微信号")
    private String wechat;

    @NotNull(message = "与子女关系为空")
    @ApiModelProperty(name="relation",value="与子女的关系")
    private RelationEnum relation;

    @ApiModelProperty(name="incomeMax",value="年收入")
    private IncomeEnum income;

    @ApiModelProperty(name="maritalStatus",value="婚姻状况")
    private MaritalStatusEnum maritalStatus;

    @ApiModelProperty(name="house",value="是否有房:0:无:1:有")
    private HouseEnum house;

    @ApiModelProperty(name="car",value="是否有车:0无:1:有")
    private CarEnum car;

    @SensitiveWords(message = "描述信息包含敏感词")
    @ApiModelProperty(name="des",value="其他描述信息")
    private String des;

    /******************************* 择偶标准 ***********************************/

    @ApiModelProperty(name="expAge",value="期望年龄")
    private AgeEnum expAge;

    @ApiModelProperty(name="expheight",value="期望身高")
    private UserHegihtEnum expheight;

    @ApiModelProperty(name="expIncome",value="期望年收入")
    private IncomeEnum expIncome;

    @ApiModelProperty(name="expCensusProvince",value="户籍地（省）")
    private String expCensusProvince;

    @ApiModelProperty(name="expCensusCity",value="户籍地（市）")
    private String expCensusCity;

    @ApiModelProperty(name="expHouse",value="期望有婚房")
    private HouseEnum expHouse;

    @ApiModelProperty(name="expEducation",value="期望对方学历")
    private EducationEnum expEducation;

    @SensitiveWords(message = "择偶说明包含敏感词")
    @ApiModelProperty(name="expDes",value="择偶的额外说明")
    private String expDes;

}
