package com.sp.blinddate.module.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.RedisServer;
import com.sp.blinddate.common.BaseLikeEnum;
import com.sp.blinddate.common.BaseReviewIenum;
import com.sp.blinddate.module.likechildinfo.service.IUserLikeChildDetailService;
import com.sp.blinddate.module.spouse.entity.SpouseRequest;
import com.sp.blinddate.module.spouse.service.ISpouseRequestService;
import com.sp.blinddate.module.statistics.entity.ProfileStatistics;
import com.sp.blinddate.module.statistics.prefix.StatisticKeysPrefix;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.entity.UserChildInfo;
import com.sp.blinddate.module.user.helper.MarriageNumberManager;
import com.sp.blinddate.module.user.helper.UserInfoIdManager;
import com.sp.blinddate.module.user.ienum.ImprovedInfoEnum;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.module.user.mapper.UserChildInfoMapper;
import com.sp.blinddate.module.user.param.UserFullInfoParam;
import com.sp.blinddate.module.user.service.IUserChildInfoService;
import com.sp.blinddate.module.user.service.IUserChildPictureService;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.blinddate.module.user.vo.ChildBlindInfoBase;
import com.sp.blinddate.module.user.vo.ChildBlindInfoDetail;
import com.sp.blinddate.utils.AliOSSUtil;
import com.sp.blinddate.utils.EnvironmentUtil;
import com.sp.blinddate.utils.TokenUtil;
import com.sp.exception.BusinessException;
import com.sp.utils.CalendarUtil;
import com.sp.utils.DateUtil;
import com.sp.utils.IdGenerateUtil;
import com.sp.utils.StringUtil;
import org.redisson.api.RBloomFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户信息表 服务实现类
 * </p>
 *
 * @author 十一
 * @since 2019-11-13
 */
@Service
@Transactional
public class UserChildInfoServiceImpl extends ServiceImpl<UserChildInfoMapper, UserChildInfo> implements IUserChildInfoService {
    private static final Logger LOG = LoggerFactory.getLogger(UserChildInfoServiceImpl.class);

    @Autowired
    private RedisServer redisServer;

    @Autowired
    private IUserService userService;

    @Autowired
    private RBloomFilter userInfoSavedFilter;

    @Autowired
    private UserChildInfoMapper userChildInfoMapper;

    @Autowired
    private IUserChildPictureService userChildPictureService;

    @Autowired
    private ISpouseRequestService spouseRequestService;

    @Autowired
    private IUserLikeChildDetailService userLikeChildDetailService;

    /**
     *功能描述 查询用户所有子女的姻缘号
     * @author 十一
     * @date 2019/12/16 15:42
     */
    @Override
    public List<String> getUserMarriages(Long userId) {
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().select(UserChildInfo::getMarriageNumber)
                .eq(UserChildInfo::getUserId, userId);
        List<UserChildInfo> list = this.list(qw);
        List<String> userMarriages = list.stream().map(UserChildInfo::getMarriageNumber).collect(Collectors.toList());
        return userMarriages;
    }

    /**
     *功能描述 根据条件查询用户信息的id
     * @author 十一
     * @date 2019/12/16 16:42
     */
    @Override
    public List<Long> getUserChildInfoIdsByCondition(Long userId) {
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().eq(UserChildInfo::getUserId, userId)
                .eq(UserChildInfo::getStatus,BaseReviewIenum.Pass)
                .select(UserChildInfo::getId);
        List<UserChildInfo> list = this.list(qw);
        List<Long> userInfos = list.stream().map(UserChildInfo::getId).collect(Collectors.toList());
        return userInfos;
    }

    /**
     *功能描述 根据姻缘号查询出对应的用户封面信息
     * @author 十一
     * @date 2019/11/30 16:45
     */
    @Override
    public List<String> getUserCoverImgByMarrages(List<String> marriages) {
        if(marriages.isEmpty()){
            return Collections.emptyList();
        }
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().in(UserChildInfo::getMarriageNumber,marriages)
                .select(UserChildInfo::getCoverImg);
        List<UserChildInfo> list = this.list(qw);
        return list.stream().map(UserChildInfo::getCoverImg).collect(Collectors.toList());
    }

    /**
     *功能描述 通过用户id获取推荐用户信息列表
     * @author 十一
     * @date 2019/11/28 11:39
     */
    @Override
    public List<UserChildInfo> getRecommandListByInfoIds(List<Long> userInfos) {
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().in(UserChildInfo::getId,userInfos);
        return this.list(qw);
    }

    @Override
    public List<UserChildInfo> getByUserId(Long id) {
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().eq(UserChildInfo::getUserId, id)
                    .orderByDesc(UserChildInfo::getUpdateTime);
        return this.list(qw);
    }

    @Override
    public List<Long> getUserIdsByMarriages(List<String> marriages) {
        if(marriages.isEmpty()){
            return Collections.emptyList();
        }
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().in(UserChildInfo::getMarriageNumber,marriages)
                .select(UserChildInfo::getUserId);
        List<UserChildInfo> list = this.list(qw);
        List<Long> result = list.stream().map(UserChildInfo::getUserId).collect(Collectors.toList());
        return result;
    }

    /**
     *功能描述 根据用户id查询出子女的姻缘号
     * @author 十一
     * @date 2019/11/20 18:02
     */
    @Override
    public List<String> getChildMarriages(Long userId) {
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        qw.lambda().eq(UserChildInfo::getUserId, userId)
                    .select(UserChildInfo::getMarriageNumber);
        List<UserChildInfo> marriageList = this.list(qw);
        List<String> result = marriageList.stream().map(UserChildInfo::getMarriageNumber).collect(Collectors.toList());
        return result;
    }

    /**
     *功能描述 获取子女的完整信息
     * @author 十一
     * @date 2019/11/20 16:20
     */
    @Override
    public ChildBlindInfoDetail getdetailInfo(Long userInfoId) {
        ChildBlindInfoBase info = userChildInfoMapper.getChildBlindInfoBase(userInfoId);
        UserChildInfo childInfo = this.getById(userInfoId);
        int likeNum = userLikeChildDetailService.countLikeNumber(childInfo.getMarriageNumber());
        if(info.getBirth() != null){
            //计算生日
            int age = CalendarUtil.getYearsFromDate(info.getBirth());
            info.setAge(age);
        }
        info.setLikeNum(likeNum);
        info.setCoverImg(childInfo.getCoverImg());
        //查询图片列表
        List<String> images = userChildPictureService.getUserChildPictures(childInfo.getId());
        SpouseRequest request = spouseRequestService.getUserRequsetByUserInfoId(childInfo.getId());
        return this.wrapperDetailInfo(childInfo, info, images, request);
    }

    /**
     *功能描述 模糊手机号
     * @author 十一
     * @date 2019/12/11 21:19
     */
    private String blurryPhone(String phone) {
        if(phone.length() == 11){
            String blurryPhone = phone.substring(0, 4) + "****" + phone.substring(7, 11);
            return blurryPhone;
        }
        return null;
    }

    /**
     *功能描述 封装detail对象
     * @author 十一
     * @date 2019/11/20 16:37
     */
    private ChildBlindInfoDetail wrapperDetailInfo(UserChildInfo childInfo,ChildBlindInfoBase info, List<String> images, SpouseRequest request) {
        ChildBlindInfoDetail detail = new ChildBlindInfoDetail();
        BeanUtils.copyProperties(info,detail);
        detail.setWechat(childInfo.getWechat());
        detail.setPhone(childInfo.getPhone());
        detail.setRelation(childInfo.getRelation()== null ? null: childInfo.getRelation().getName());
        detail.setIncome(childInfo.getIncome()== null ? null:childInfo.getIncome().getName());
        detail.setHouse(childInfo.getHouse()== null ? null:childInfo.getHouse().getName());
        detail.setCar(childInfo.getCar()== null ? null: childInfo.getCar().getName());
        detail.setDes(childInfo.getDes());
        detail.setImages(images);
        detail.setRequestAge(request.getAge() == null ? null:request.getAge().getName());
        detail.setRequestCensusProvince(request.getCensusProvince());
        detail.setRequestCensusCity(request.getCensusCity());
        detail.setRequestDes(request.getDes());
        detail.setRequestEducation(request.getEducation() == null ? null: request.getEducation().getName());
        detail.setRequestHeight(request.getHeight() == null ? null: request.getHeight().getName());
        detail.setRequestHouse(request.getHouse() == null ? null: request.getHouse().getName());
        detail.setRequestIncome(request.getIncome() == null ? null: request.getIncome().getName());
        detail.setEducation(childInfo.getEducation()== null ? null: childInfo.getEducation().getName());
        detail.setHeight(childInfo.getHeight()== null ? null: childInfo.getHeight().getName());
        detail.setMaritalStatus(childInfo.getMaritalStatus()== null ? null:childInfo.getMaritalStatus().getName());
        detail.setUserInfoId(childInfo.getId());
        detail.setGender(childInfo.getGender()== null ? null:childInfo.getGender().getName());
        detail.setCarrer(childInfo.getCarrer() == null ? null : childInfo.getCarrer().getName());
        return detail;
    }

    /**
     *功能描述 获取子女的基本信息
     * @author 十一
     * @date 2019/11/20 15:27
     */
    @Override
    public List<ChildBlindInfoBase> getBaseInfo(User user) {
        List<ChildBlindInfoBase> list = new ArrayList<ChildBlindInfoBase>();
        //查询所有子女的信息
        List<UserChildInfo> childInfoList = this.getByUserId(user.getId());
        for (UserChildInfo childInfo : childInfoList) {
            ChildBlindInfoBase info = userChildInfoMapper.getChildBlindInfoBase(childInfo.getId());
            int likeNum = userLikeChildDetailService.countLikeNumber(childInfo.getMarriageNumber());
            if(info.getBirth() != null){
                //计算生日
                int age = CalendarUtil.getYearsFromDate(info.getBirth());
                info.setAge(age);
            }
            info.setLikeNum(likeNum);
            info.setCoverImg(childInfo.getCoverImg());
            info.setEducation(childInfo.getEducation()== null ? null: childInfo.getEducation().getName());
            info.setHeight(childInfo.getHeight() == null ? null : childInfo.getHeight().getName());
            info.setIncome(childInfo.getIncome() == null ? null : childInfo.getIncome().getName());
            info.setUserInfoId(childInfo.getId());
            info.setCarrer(childInfo.getCarrer() == null ? null : childInfo.getCarrer().getName());
            info.setGender(childInfo.getGender().getName());
            list.add(info);
        }
        return list;
    }

    /**
     * 功能描述 保存用户的信息
     *
     * @author 十一
     * @date 2019/11/19 16:03
     */
    @Override
    public void saveInfo(User user, UserFullInfoParam fullInfoParam) {
        Boolean result = redisServer.setNx(UserKeyPrefix.userSaveInfoLock, user.getId() + "", user.getId() + "", 30);
        //如果能够获取到key，那么允许用户保存信息，否则可以认定用户重复提交
        if(!result){
            return;
        }
        UserChildInfo info = this.updateUserChildInfo(user, fullInfoParam);
        userChildPictureService.updateUserChildPictures(info.getId(), fullInfoParam);
        spouseRequestService.updateSpouseRequest(info.getId(), fullInfoParam);
        Integer processCount = this.countUserInfoProcess(fullInfoParam);
        info.setWechat(fullInfoParam.getWechat());
        info.setPhone(fullInfoParam.getPhone());
        user.setInfoProcess(processCount);
        //更新信息
        redisServer.set(UserKeyPrefix.userInfoLoginToken, TokenUtil.getToken(), user);
        userService.updateById(user);
        user.setImprovedInfo(ImprovedInfoEnum.Improved);
        redisServer.set(UserKeyPrefix.userInfoLoginToken, TokenUtil.getToken(), user);
        //判断是否发布过
        boolean hasPublish = userInfoSavedFilter.contains(info.getId());
        if(!hasPublish){
            Date today = new Date();
            String todayStr = DateUtil.dateStrDay(today);
            //保存今日发布数
            ProfileStatistics ps = redisServer.get(StatisticKeysPrefix.profileStatistics, todayStr, ProfileStatistics.class);
            if(ps == null){
                ps = new ProfileStatistics();
                ps.setId(IdGenerateUtil.generateId());
                ps.setDate(new Date());
            }
            ps.setTodayPublishTotal(ps.getTodayPublishTotal() + 1);
            ps.setPublishTotal(ps.getPublishTotal() + 1);
            redisServer.set(StatisticKeysPrefix.profileStatistics, todayStr, ps);
        }
    }

    /**
     * 功能描述 计算用户的信息完成度
     *
     * @author 十一
     * @date 2019/11/19 17:30
     */
    private Integer countUserInfoProcess(UserFullInfoParam fullInfoParam) {
        Class<? extends UserFullInfoParam> clazz = fullInfoParam.getClass();
        int fieldsCount = clazz.getDeclaredFields().length;
        Method[] methods = clazz.getDeclaredMethods();
        Integer count = 0;
        try {
            for (Method method : methods) {
                if (method.getName().startsWith("get")) {
                    if (method.invoke(fullInfoParam) != null) {
                        count = count + 1;
                    }
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return (int) (((float) count / (float) fieldsCount) * 100);
    }

    /**
     * 功能描述 更新用户子女信息
     *
     * @author 十一
     * @date 2019/11/19 16:47
     */
    private UserChildInfo updateUserChildInfo(User user, UserFullInfoParam fullInfoParam) {
        //判断是否是用户新增的子女信息
        UserChildInfo info = null;
        if(fullInfoParam.getUserInfoId() == null){
            info = new UserChildInfo();
            info.setUserId(user.getId());
            info.setId(UserInfoIdManager.generatorId());
            info.setMarriageNumber(MarriageNumberManager.generatorMarriageNumber());
        }else{
            info = this.getById(fullInfoParam.getUserInfoId());
        }
        info.setGender(fullInfoParam.getGender());
        info.setDateOfBirth(fullInfoParam.getDateOfBirth());
        info.setResidenceProvince(fullInfoParam.getResidenceProvince());
        info.setResidenceCity(fullInfoParam.getResidenceCity());
        info.setCensusProvince(fullInfoParam.getCensusProvince());
        info.setCensusCity(fullInfoParam.getCensusCity());
        info.setEducation(fullInfoParam.getEducation());
        info.setHeight(fullInfoParam.getHeight());
        info.setCarrer(fullInfoParam.getCarrer());
        info.setRelation(fullInfoParam.getRelation());
        info.setIncome(fullInfoParam.getIncome());
        info.setMaritalStatus(fullInfoParam.getMaritalStatus());
        info.setHouse(fullInfoParam.getHouse());
        info.setUpdateTime(new Date());
        info.setCar(fullInfoParam.getCar());
        info.setDes(fullInfoParam.getDes());
        info.setStatus(BaseReviewIenum.Review);
        info.setCoverImg(fullInfoParam.getCoverImg());
        info.setWechat(fullInfoParam.getWechat());
        info.setPhone(fullInfoParam.getPhone());
        if(fullInfoParam.getUserInfoId() == null){
            this.save(info);
        }else{
            this.updateById(info);
        }
        return info;
    }

    /**
     * 功能描述 上传文件到aliyun 并返回对应的url
     *
     * @author 十一
     * @date 2019/11/15 11:20
     */
    @Override
    public String uploadUserPic(MultipartFile pic) {
        String userId = StringUtil.uuid();
        String key = EnvironmentUtil.getProperty("oss.blinddate") + "/" + userId +
                "_" + DateUtil.dateStrToday() +
                "_" + StringUtil.uuid() +
                StringUtil.getSuffixByFileName(pic.getOriginalFilename());
        try {
            String uri = AliOSSUtil.uploadByBytes(pic.getBytes(), key);
            redisServer.set(UserKeyPrefix.userInfoPicUpload, key, 1);
            return uri;
        } catch (IOException e) {
            LOG.error("文件上传失败：错误原因是{}", e.getMessage());
            throw new BusinessException("文件上传失败！");
        }
    }
}
