package com.sp.blinddate.module.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.sp.RedisServer;
import com.sp.blinddate.base.status.StatusEnum;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.entity.UserChildPicture;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.module.user.mapper.UserChildPictureMapper;
import com.sp.blinddate.module.user.param.UserFullInfoParam;
import com.sp.blinddate.module.user.service.IUserChildPictureService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sp.blinddate.utils.EnvironmentUtil;
import com.sp.utils.IdGenerateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author 十一
 * @since 2019-11-13
 */
@Service
@Transactional
public class UserChildPictureServiceImpl extends ServiceImpl<UserChildPictureMapper, UserChildPicture> implements IUserChildPictureService {

    @Autowired
    private RedisServer redisServer;

    /**
     *功能描述 获取所有的图片
     * @author 十一
     * @date 2019/11/20 16:22
     */
    @Override
    public List<String> getUserChildPictures(Long userInfoid) {
        QueryWrapper<UserChildPicture> qw = new QueryWrapper<>();
        qw.lambda().eq(UserChildPicture::getUserInfoId, userInfoid)
                    .orderByAsc(UserChildPicture::getOrders);
        List<UserChildPicture> list = this.list(qw);
        List<String> result = list.stream().map(UserChildPicture::getPictureUrl).collect(Collectors.toList());
        return result;
    }



    /**
     *功能描述 保存用户子女的照片
     * @author 十一
     * @date 2019/11/19 16:54
     */
    @Override
    public void updateUserChildPictures(Long userInfoId, UserFullInfoParam fullInfoParam) {
        UpdateWrapper<UserChildPicture> uw = new UpdateWrapper<>();
        uw.lambda().eq(UserChildPicture::getUserInfoId, userInfoId);
        this.remove(uw);
        List<String> images = fullInfoParam.getImages();
        if(images.isEmpty()){
            return;
        }
        //删除之前的图片，防止保存出错
        List<UserChildPicture> list = new ArrayList<>();
        UpdateWrapper<UserChildPicture> duw = new UpdateWrapper<>();
        duw.lambda().eq(UserChildPicture::getUserInfoId, userInfoId);
        this.remove(duw);

        for (int i = 0; i < images.size(); i++) {
            UserChildPicture pic = new UserChildPicture();
            pic.setUserInfoId(userInfoId);
            pic.setPictureUrl(images.get(i));
            pic.setId(IdGenerateUtil.generateId());
            pic.setStatus(StatusEnum.Normal);
            pic.setOrders(i);
            //删除某个redis键
            String keyPrefix = "https://" + EnvironmentUtil.getProperty("oss.bucketname") + "." + EnvironmentUtil.getProperty("oss.endpoint") + "/";
            String key = images.get(i).replace(keyPrefix, "");
            redisServer.delete(UserKeyPrefix.userInfoPicUpload, key);
            list.add(pic);
        }
        this.saveBatch(list);
    }
}
