package com.sp.blinddate.module.user.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sp.RedisServer;
import com.sp.blinddate.base.status.VIPEnum;
import com.sp.blinddate.module.likechildinfo.service.IUserLikeChildDetailService;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.helper.UserIdManager;
import com.sp.blinddate.module.user.ienum.ImprovedInfoEnum;
import com.sp.blinddate.module.user.ienum.UserStatusEnum;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.module.user.mapper.UserMapper;
import com.sp.blinddate.module.user.param.EncryUserInfoParam;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.blinddate.module.user.vo.*;
import com.sp.blinddate.module.vip.param.BaseParam;
import com.sp.blinddate.module.vip.vo.UserVO;
import com.sp.blinddate.utils.EnvironmentUtil;
import com.sp.blinddate.utils.TokenUtil;
import com.sp.blinddate.utils.WxUtils;
import com.sp.exception.BusinessException;
import com.sp.utils.OkHttpUtil;
import com.sp.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.swing.text.ParagraphView;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户账户表 服务实现类
 * </p>
 *
 * @author 十一
 * @since 2019-11-13
 */
@Service
@Transactional
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    private static final Logger LOG = LoggerFactory.getLogger(UserServiceImpl.class);

    @Autowired
    private RedisServer redisServer;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private IUserLikeChildDetailService userLikeChildDetailService;

    /**
     *功能描述 解析用户绑定微信的手机号码
     * @author 十一
     * @date 2019/11/20 11:57
     */
    @Override
    public String getUserBindPhone(User user, EncryUserInfoParam param) {
        UserInfo userInfoVO = redisServer.get(UserKeyPrefix.userWechatInfo, param.getOpenId(), UserInfo.class);
        try {
            String result = WxUtils.decrypt(userInfoVO.getSessionKey(), param.getEncryptedData(), param.getIv());
            WechatPhoneVO vo = JSON.parseObject(result, WechatPhoneVO.class);
            String phone = vo.getPurePhoneNumber();
            //保存用户手机号
            UpdateWrapper<User> uw = new UpdateWrapper<>();
            uw.lambda().set(User::getPhone, phone)
                    .eq(User::getId, user.getId());
            this.update(uw);
            String token = TokenUtil.getToken();
            redisServer.set(UserKeyPrefix.userInfoLoginToken, token, user);
            return phone;
        } catch (Exception e) {
            LOG.error("获取微信绑定的手机号异常,原因是{}",e.getMessage());
            throw new BusinessException("获取绑定手机号异常");
        }
    }

    @Override
    public PageInfo<UserVO> getUserListOrderByCreateTime(BaseParam param) {
        PageHelper.startPage(param.getPageNum(), param.getPageSize());
        List<UserVO> list = userMapper.getUserListOrderByCreateTime(param);
        for (UserVO vo : list) {
            vo.setVip(vo.getVipStatus().getName());
            vo.setUserId(vo.getId());
        }
        PageInfo info = new PageInfo(list);
        return info;
    }

    /**
     *功能描述 查询用户的头像列表
     * @author 十一
     * @date 2019/11/20 19:18
     */
    @Override
    public List<String> getUserHeadImgsByIds(List<Long> userIds) {
        if(userIds.isEmpty()){
            return Collections.EMPTY_LIST;
        }
        QueryWrapper<User> qw = new QueryWrapper<>();
        qw.lambda().in(User::getId, userIds).select(User::getHeadImg);
        List<User> list = this.list(qw);
        List<String> result = list.stream().map(User::getHeadImg).collect(Collectors.toList());
        return result;
    }

    @Override
    public MineInfoVO getMineInfo(User user) {
        String phone = user.getPhone();
        int likeNum = 0;
        int count = userLikeChildDetailService.countLikeNumberByUser(user.getId());
        likeNum = count;
        //喜欢我的用户列表
        List<String> likeMychildUserList = userLikeChildDetailService.getLikeMyChildListByThumbnail(user.getId());
        //我喜欢的用户列表
        List<String> likedUserList = userLikeChildDetailService.getLikedChildListByThumbnail(user.getId());
        MineInfoVO vo = new MineInfoVO();
        vo.setLikedUserList(likedUserList);
        vo.setBeLikedUserList(likeMychildUserList);
        vo.setLikeNum(likeNum);
        vo.setPhone(phone);
        vo.setHeadImg(user.getHeadImg());
        vo.setNickName(user.getNickName());
        vo.setUserId(user.getId());
        vo.setVIP(user.getVip() == VIPEnum.NormalVip);
        vo.setVipExpireTime(user.getVipExpireDate());
        return vo;
    }

    /**
     *功能描述 用户登陆
     * @author 十一
     * @date 2019/11/18 17:02
     * @param param
     */
    @Override
    public UserLoginVO login(EncryUserInfoParam param) {
       UserInfo userInfoVO = redisServer.get(UserKeyPrefix.userWechatInfo, param.getOpenId(), UserInfo.class);
        String token = null;
        String result = null;
        try{
            result = WxUtils.decrypt(userInfoVO.getSessionKey(), param.getEncryptedData(), param.getIv());
            WechatInfoVO wechat = JSON.parseObject(result, WechatInfoVO.class);
            //判断是否需要注册
            User user = this.checkUserExist(wechat.getOpenId());
            if(user == null){
                //注册新用户
                user = this.wrapperUser(wechat);
                this.save(user);
            }
            //生成用户token
            token = StringUtil.getRandomStrWithLength(50);
            //判断用户之前是否有登陆，如果有登陆，删除之前的缓存
            String oldToken = TokenUtil.getToken();
            if(oldToken != null){
                redisServer.delete(UserKeyPrefix.userInfoLoginToken, oldToken);
            }
            redisServer.set(UserKeyPrefix.userInfoLoginToken, token, user);
            UserLoginVO vo = new UserLoginVO();
            vo.setToken(token);
            vo.setVIP(user.getVip() == VIPEnum.NormalVip);
            vo.setVipExpireTime(user.getVipExpireDate());
            vo.setVipPrice(redisServer.get(UserKeyPrefix.userVipPrivaliagePrice,"vip", BigDecimal.class));
            return vo;
        } catch (Exception e) {
            LOG.error("微信登陆异常,原因是{}",e.getMessage());
            throw new BusinessException("微信登陆异常");
        }
    }

    /**
     *功能描述 包装用户信息
     * @author 十一
     * @date 2019/11/19 15:01
     */
    private User wrapperUser(WechatInfoVO wechat) {
        User user = new User();
        user.setHeadImg(wechat.getAvatarUrl());
        user.setImprovedInfo(ImprovedInfoEnum.UnImproved);
        user.setId(UserIdManager.generatorId());
        user.setNickName(wechat.getNickName());
        user.setUnionId(null);
        user.setCreateTime(new Date());
        user.setProvince(wechat.getProvince());
        user.setCity(wechat.getCity());
        user.setCountry(wechat.getCountry());
        user.setStatus(UserStatusEnum.Normal);
        user.setOpenId(wechat.getOpenId());
        user.setVip(VIPEnum.Normal);
        return user;
    }

    /**
     *功能描述 判断用户是否存在
     * @author 十一
     * @date 2019/11/19 14:55
     */
    private User checkUserExist(String openId) {
        QueryWrapper<User> qw = new QueryWrapper<>();
        qw.lambda().eq(User::getOpenId, openId);
        User user = this.getOne(qw);
        if(user != null && user.getStatus().equals(UserStatusEnum.Banned)) {
            throw new BusinessException("用户已经被封");
        }
        return user;
    }

    /**
     *功能描述 根据传入的凭证去换取
     * @author 十一
     * @date 2019/11/15 16:38
     */
    @Override
    public String loginByCode(String code) {
        try{
            String appId = EnvironmentUtil.getProperty("wechat.appId");
            String appSecret = EnvironmentUtil.getProperty("wechat.appSecret");
            String url = "https://api.weixin.qq.com/sns/jscode2session";
            Map<String,String> map = new HashMap<>();
            map.put("appid", appId);
            map.put("secret", appSecret);
            map.put("js_code", code);
            map.put("grant_type", "authorization_code");
            JSONObject response = OkHttpUtil.getRequest(url, map);
            String openId = response.get("openid").toString();
            String sessionKey = response.get("session_key").toString();
            UserInfo vo = new UserInfo();
            vo.setOpenId(openId);
            vo.setSessionKey(sessionKey);
            //将用户信息缓存到redis
            redisServer.set(UserKeyPrefix.userWechatInfo, openId, vo);
            return openId;
        }catch(Exception e){
            e.printStackTrace();
        }
        return null;
    }
}
