package com.sp.blinddate.schedultask;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.sp.RedisServer;
import com.sp.blinddate.base.status.VIPEnum;
import com.sp.blinddate.module.statistics.entity.ProfileStatistics;
import com.sp.blinddate.module.statistics.prefix.StatisticKeysPrefix;
import com.sp.blinddate.module.statistics.service.IProfileStatisticsService;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.ienum.VipExpireStatusEnum;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.utils.CalendarUtil;
import com.sp.utils.DateUtil;
import com.sp.utils.IdGenerateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @Author: 十一
 * @Date: 2019/9/4 16:35
 * @Descrption 定时器
 **/
@Component
public class ScheduleTask {

    private static final Logger LOG = LoggerFactory.getLogger(ScheduleTask.class);

    @Autowired
    private IUserService userService;

    @Autowired
    private RedisServer redisServer;

    @Autowired
    private IProfileStatisticsService profileStatisticsService;

    /**
     *功能描述 重置用户的特权,每小时执行一次
     * @author 十一
     * @date 2019/11/2 14:54
     */
    @Scheduled(cron = "0 0 0/1 * * ?")
    public void checkUserPrivilege(){
        Date now = new Date();
        //获取到一个小时后的时间
        Date anHourLaterTime = CalendarUtil.addHour(now, 1);
        //查询出一个小时候就会过期的VIP用户列表
        QueryWrapper<User> qwVIP = new QueryWrapper<>();
        qwVIP.lambda().lt(User::getVipExpireDate, anHourLaterTime);
        List<User> list = userService.list(qwVIP);
        for (User user : list) {
            user.setVip(VIPEnum.Normal);
            user.setVipExpireStatus(VipExpireStatusEnum.Expired);
        }
        if(!list.isEmpty()){
            userService.updateBatchById(list);
        }
        LOG.info("{}个用户的vip特权过期",list.size());
    }

    /**
     *功能描述 晚上12：00将前一天的统计信息保存到mysql
     * @author 十一
     * @date 2019/11/2 14:54
     */
    @Scheduled(cron = "0 0 0 * * ?")
    public void saveProfileStatistics(){
        Date today = new Date();
        Date yestoday = CalendarUtil.getDate(today, -1);
        String yesTodayStr = DateUtil.dateStrDay(yestoday);
        ProfileStatistics ps = redisServer.get(StatisticKeysPrefix.profileStatistics,
                yesTodayStr, ProfileStatistics.class);
        if(ps == null) {
            ps = new ProfileStatistics();
            ps.setDate(yestoday);
            ps.setId(IdGenerateUtil.generateId());
            ps.setPublishTotal(0);
            ps.setTodayCertificationFee(BigDecimal.ZERO);
            ps.setYestodayPublishTotal(0);
            ps.setTodayPublishTotal(0);
            ps.setTotalCertificationFee(BigDecimal.ZERO);
            ps.setYestodayCertificationFee(BigDecimal.ZERO);
        }
        //保存昨日的统计信息
        profileStatisticsService.save(ps);
        ProfileStatistics todayPS = new ProfileStatistics();
        todayPS.setYestodayCertificationFee(ps.getTodayCertificationFee());
        todayPS.setYestodayPublishTotal(ps.getTodayPublishTotal());
        todayPS.setTotalCertificationFee(ps.getTotalCertificationFee());
        todayPS.setPublishTotal(ps.getPublishTotal());
        todayPS.setDate(today);
        String todayStr = DateUtil.dateStrDay(today);
        redisServer.set(StatisticKeysPrefix.profileStatistics, todayStr, todayPS);
    }
}
