package com.sp.blinddate.utils;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.OSSObjectSummary;
import com.aliyun.oss.model.ObjectListing;
import com.sp.exception.MusicException;
import com.sp.utils.HttpUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: 十一
 * @Date: 2019/8/29 23:13
 * @Descrption 阿里oss工具包
 **/
@Configuration
public class AliOSSUtil {

    private static final Logger LOG = LoggerFactory.getLogger(AliOSSUtil.class);

    // Endpoint以杭州为例，其它Region请按实际情况填写。

    public static String ENDPOINT;

    public static String ACCESSKEYID;

    public static String ACCESSKEYSECRET;

    public static String BUCKETNAME;

    public static String DEFAULTHEADIMG;

    public static String BACKGROUNDMUSIC;

    public static String USERREALFACE;

    @Value("${oss.userRealFace}")
    public void setUSERREALFACE(String USERREALFACE) {
        AliOSSUtil.USERREALFACE = USERREALFACE;
    }

    @Value("${oss.endpoint}")
    public void setENDPOINT(String ENDPOINT) {
        AliOSSUtil.ENDPOINT = ENDPOINT;
    }

    @Value("${oss.accesskeyid}")
    public void setACCESSKEYID(String ACCESSKEYID) {
        AliOSSUtil.ACCESSKEYID = ACCESSKEYID;
    }

    @Value("${oss.accesskeysecret}")
    public void setACCESSKEYSECRET(String ACCESSKEYSECRET) {
        AliOSSUtil.ACCESSKEYSECRET = ACCESSKEYSECRET;
    }

    @Value("${oss.bucketname}")
    public void setBUCKETNAME(String BUCKETNAME) {
        AliOSSUtil.BUCKETNAME = BUCKETNAME;
    }

    @Value("${oss.defaultHeadImg}")
    public void setDEFAULTHEADIMG(String DEFAULTHEADIMG) {
        AliOSSUtil.DEFAULTHEADIMG = DEFAULTHEADIMG;
    }

    @Value("${oss.backgroundMusic}")
    public void setBACKGROUNDMUSIC(String BACKGROUNDMUSIC) {
        AliOSSUtil.BACKGROUNDMUSIC = BACKGROUNDMUSIC;
    }


    /**
     * 功能描述 上传文件，返回路径
     *
     * @return java.lang.String
     * @author 十一
     * @date 2019/9/6 11:22
     */
    public static String upload(File file, String key) {
        doUpload(file,null,key);
        return "https://" + BUCKETNAME + "." + ENDPOINT + "/" + key;
    }


    /**
     *功能描述 根据url上传到ali oss并返回地址
     * @author 十一
     * @date 2019/11/4 10:46
     */
    public static String uploadByUrl(String url, String key) {
        // 创建OSSClient实例。
        byte[] bytes = HttpUtil.getBytesByUrl(url);
        doUpload(null,bytes,key);
        return "https://" + BUCKETNAME + "." + ENDPOINT + "/" + key;
    }


    /**
     *功能描述 通过字节数据上传文件
     * @author 十一
     * @date 2019/10/16 10:22
     */
    public static String uploadByBytes(byte[] bytes,String key){
        doUpload(null,bytes,key);
        return "https://" + BUCKETNAME + "." + ENDPOINT + "/" + key;
    }

    /**
     *功能描述 对象上传
     * @author 十一
     * @date 2019/11/4 11:03
     */
    public static void doUpload(File file,byte[] bytes,String key){
        InputStream inputStream = null;
        OSS ossClient = getClient();
        try {
            if(file != null){
                inputStream = new FileInputStream(file);
            }else{
                inputStream = new ByteArrayInputStream(bytes);
            }
            ossClient.putObject(BUCKETNAME, key, inputStream);
        } catch (Exception e) {
            LOG.error("文件上传失败：{}", e.getMessage());
            throw new MusicException("文件上传异常");
        } finally {
            ossClient.shutdown();
        }
    }

    /**
     *功能描述 删除对象
     * @author 十一
     * @param key
     * @return void
     * @date 2019/9/6 15:28
     */
    public static void deleteObject(String key){
        OSS client = getClient();
        if(exist(client,key)){
            client.deleteObject(BUCKETNAME,key);
        }
        client.shutdown();
    }

    /**
     *功能描述 获取ossClient对象
     * @author 十一
     * @param
     * @return com.aliyun.oss.OSS
     * @date 2019/9/6 15:26
     */
    public static OSS getClient(){
        return new OSSClientBuilder().build(ENDPOINT, ACCESSKEYID, ACCESSKEYSECRET);
    }

    /**
     * 功能描述 判断是否存在
     * @param ossClient
     * @param key
     * @return boolean
     * @author 十一
     * @date 2019/9/6 15:20
     */
    public static boolean exist(OSS ossClient, String key) {
        return ossClient.doesObjectExist(BUCKETNAME, key);
    }


    /**
     *功能描述 根据url匹配前缀获取key
     * @author 十一
     * @param url
     * @return java.lang.String
     * @date 2019/9/6 15:29
     */
    public static String getKey(String url){
        String prefix = "https://" + BUCKETNAME + "." + ENDPOINT + "/";
        return url.replace(prefix, "");
    }

    /**
     * 功能描述 获取所有默认的头像列表
     *
     * @param
     * @return java.util.List<java.lang.String>
     * @author 十一
     * @date 2019/8/29 23:17
     */
    public static List<String> getDefaultHeadFileKey() {
        // 创建OSSClient实例。
        OSS ossClient = getClient();
        // 列举文件。 如果不设置KeyPrefix，则列举存储空间下所有的文件。KeyPrefix，则列举包含指定前缀的文件。
        ObjectListing objectListing = ossClient.listObjects(BUCKETNAME, DEFAULTHEADIMG);
        List<OSSObjectSummary> sums = objectListing.getObjectSummaries();
        List<String> headImgList = new ArrayList<>();
        for (OSSObjectSummary s : sums) {
            headImgList.add("https://" + BUCKETNAME + "." + ENDPOINT + "/" + s.getKey());
        }

        if (headImgList.size() == 0) {
            headImgList.add("https://" + BUCKETNAME + "." + ENDPOINT + "/" + DEFAULTHEADIMG + "/defaultHeadImg.jpeg");
        }
        if (headImgList.size() > 1) {
            headImgList.remove(0);
        }
        // 关闭OSSClient。
        ossClient.shutdown();
        return headImgList;
    }

    /**
     * 功能描述 如果上传了默认头像则刷新
     *
     * @param
     * @return void
     * @author 十一
     * @date 2019/8/29 23:57
     */
    public static void refreshHeadList() {
        getDefaultHeadFileKey();
    }
}
