package com.sp.blinddate.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.Serializable;

/**
 * 渠道来源#客户端机型型号#后台接口版本#客户端UI版本
 */
public class UAParser {
	
	private static final Logger LOG = LoggerFactory.getLogger(UAParser.class);

	public static UserAgent doParseUA(HttpServletRequest req) {
		String uaStr = req.getHeader("My-User-Agent");
		if(uaStr == null || uaStr.split("#").length != 4) {
			LOG.warn("UA fail, UA {}", uaStr);
			return null;
		}
		String[] uas = uaStr.split("#");
		String channel = uas[0];
		String client = uas[1];
		int apiVersion = 0;
		try{
			apiVersion = Integer.valueOf(uas[2]);
		} catch(Exception e) {
			LOG.warn("api version fail, UA {}", uaStr);
			return null;
		}
		int appVersion = 0;
		try{
			appVersion = Integer.valueOf(uas[3]);
		} catch(Exception e) {
			LOG.warn("app version fail, UA {}", uaStr);
			return null;
		}
		String ip = WebUtil.getClientIp(req);
		if(StringUtils.isEmpty(ip)) {
			ip = "0.0.0.0";
		}
		UserAgent ua = new UserAgent();
		ua.setChannel(channel);
		ua.setClient(client);
		ua.setApiVersion(apiVersion);
		ua.setAppVersion(appVersion);
		ua.setIp(ip);
		return ua;
	}
	
	public static class UserAgent implements Serializable {
		
		private static final long serialVersionUID = 1L;
		private String channel;
		private String client;
		private int apiVersion;
		private int appVersion;
		private String ip;
		
		public String getChannel() {
			return channel;
		}
		public void setChannel(String channel) {
			this.channel = channel;
		}
		public String getClient() {
			return client;
		}
		public void setClient(String client) {
			this.client = client;
		}
		public int getApiVersion() {
			return apiVersion;
		}
		public void setApiVersion(int apiVersion) {
			this.apiVersion = apiVersion;
		}
		public int getAppVersion() {
			return appVersion;
		}
		public void setAppVersion(int appVersion) {
			this.appVersion = appVersion;
		}
		public String getIp() {
			return ip;
		}
		public void setIp(String ip) {
			this.ip = ip;
		}
		
		public boolean isAndroid() {
			return "Android".equalsIgnoreCase(this.channel);
		}
		
		public boolean isiOS() {
			return "iOS".equalsIgnoreCase(this.channel);
		}
	}
}
