package com.sp.blinddate.utils;

import com.sp.utils.JsonUtil;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class WebUtil {
	
	public static void respText(HttpServletResponse resp, String text) throws IOException {
		resp.setContentType("text/html");
		resp.setCharacterEncoding("UTF-8");
		ServletOutputStream outputStream = resp.getOutputStream();
		outputStream.write(text.getBytes());
		outputStream.flush();
	}

	public static void respJson(HttpServletResponse resp, String json) throws IOException {
		resp.setContentType("application/json");
		resp.setCharacterEncoding("UTF-8");
		ServletOutputStream outputStream = resp.getOutputStream();
		JsonUtil.MAPPER.writeValue(outputStream, json);
		outputStream.flush();
	}
	

	public static void respXml(HttpServletResponse resp, String xml) throws IOException {
		resp.setContentType("application/mapper");
		resp.setCharacterEncoding("UTF-8");
		ServletOutputStream outputStream = resp.getOutputStream();
		outputStream.write(xml.getBytes());
		outputStream.flush();
	}
	
	public static void respJsonObject(HttpServletResponse resp, Object obj) throws IOException {
		resp.setContentType("application/json");
		resp.setCharacterEncoding("UTF-8");
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		JsonUtil.MAPPER.writeValue(baos, obj);
		ServletOutputStream outputStream = resp.getOutputStream();
		baos.writeTo(outputStream);
		outputStream.flush();
	}
	
	public static String urlEncode(String str, String charset) {
		String ret = "";
		try {
			ret = URLEncoder.encode(str, charset);
		} catch (UnsupportedEncodingException e) {
			ret = str;
		}
		return ret;
	}
	
	public static String getClientIp(HttpServletRequest request) {
		String clientIP = "";
		String strClientIPList; 
		strClientIPList = request.getHeader("X-Forwarded-For");
		if (StringUtils.isBlank(strClientIPList)) {
			strClientIPList = request.getHeader("X-Real-IP");
		}
		if(StringUtils.isNotBlank(strClientIPList)) {
			if(strClientIPList.indexOf(",")>-1) {
				clientIP = strClientIPList.substring(0, strClientIPList.indexOf(","));
			} else {
				clientIP = strClientIPList;
			}
		}
		
		if(StringUtils.isBlank(clientIP)) {
			strClientIPList = request.getHeader("PROXY_FORWARDED_FOR");
			if(StringUtils.isBlank(strClientIPList)) {
				clientIP = request.getRemoteAddr();
			} else {
				if(strClientIPList.indexOf(",")>-1) {
					clientIP = strClientIPList.substring(0, strClientIPList.indexOf(","));
				} else {
					clientIP = strClientIPList;
				}
			}
		}
		if(!ipCheck(clientIP)) {
			clientIP = request.getRemoteAddr();
		}
		return clientIP;
	}
	
	private static boolean ipCheck(String text) {
        if (text != null && !text.isEmpty()) {
            String regex = "^(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|[1-9])\\."
                      +"(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)\\."
                      +"(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)\\."
                      +"(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)$";
            if (text.matches(regex)) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }
	
	public static Map<String, String> getParameterMap(HttpServletRequest request) {
		Map<String, String> params = new HashMap<String, String>();
		Map<String, String[]> requestParams = request.getParameterMap();
		for (Iterator<String> iter = requestParams.keySet().iterator(); iter.hasNext();) {
			String name = (String) iter.next();
			String[] values = (String[]) requestParams.get(name);
			String valueStr = "";
			for (int i = 0; i < values.length; i++) {
				valueStr = (i == values.length - 1) ? valueStr + values[i] : valueStr + values[i] + ",";
			}
			params.put(name, valueStr);
		}
		return params;
	}
	
	public static String getParameterStr(Map<String, String> requestParams) {
		if(requestParams==null || requestParams.size()<1) {
			return "";
		}
		StringBuilder paramStrBuf = new StringBuilder(requestParams.size()*(10+20));
		for (Iterator<String> iter = requestParams.keySet().iterator(); iter.hasNext();) {
			String name = (String) iter.next();
			String value = (String) requestParams.get(name);
			paramStrBuf.append(name).append("=").append(value).append("&");
		}
		paramStrBuf.deleteCharAt(paramStrBuf.length()-1);
		return paramStrBuf.toString();
	}
	
	/**
	 * 获取提交的参数
	 * @param request
	 * @return String
	 * @throws IOException
	 */
	public static String getParamFromStream(HttpServletRequest request) throws IOException {
		StringBuilder stringBuilder = new StringBuilder();
		String line = null;
		InputStream inputStream = request.getInputStream();
		BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
		while((line = bufferedReader.readLine()) != null){
            stringBuilder.append(line);
        }
		return stringBuilder.toString();
	}
	
	public static boolean isIOS(HttpServletRequest request) {
		String UserAgent = request.getHeader("User-Agent");
		if (UserAgent.startsWith("iOS")) {
			return true;
		}
		return false;
    }
	
	public static boolean isAjax(HttpServletRequest request) {
		return "XMLHttpRequest".equals(request.getHeader("x-requested-with"));
	}
	
	private WebUtil() {}
}
