package com.sp.blinddate.web.converter;

import com.sp.ienum.IEnum;
import com.sp.blinddate.utils.PackageScanner;
import com.sp.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.ConverterFactory;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * @author apple
 */
@SuppressWarnings({ "rawtypes", "unchecked" })
public class EnumConverterFactory implements ConverterFactory<String, IEnum>, InitializingBean {

	private static final Logger LOG = LoggerFactory.getLogger(EnumConverterFactory.class);
	private static final Map<Class<? extends IEnum>, Converter<String, ? extends IEnum>> CACHE = new HashMap<>();

	private String basePackage;
	
	@Override
	public <T extends IEnum> Converter<String, T> getConverter(
			Class<T> targetType) {
		Converter converter = CACHE.get(targetType);
		return converter;
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		LOG.info("enumConverterFactory packageScan {} begin", basePackage);
		if(StringUtil.isBlank(basePackage)) {
			return;
		}
		String[] packages = basePackage.split(",");
		Collection<Class<?>> clazzs = PackageScanner.scanPackages(packages, new PackageScanner.Filter(){
			@Override
			public boolean filter(Class<?> clazz) {
				if(IEnum.class.isAssignableFrom(clazz) && clazz.isEnum()) {
					return true;
				}
				return false;
			}
		});
		for(Class<?> c : clazzs) {
			Class<? extends IEnum> clazz = (Class<? extends IEnum>)c;
			EnumConverter<? extends IEnum> converter = new EnumConverter(clazz);
			CACHE.putIfAbsent(clazz, converter);
			LOG.debug("enumConverterFactory {} - {}", clazz.getSimpleName(), converter);
		}
		LOG.info("enumConverterFactory packageScan {} - {} end", basePackage, clazzs.size());
	}

	public String getBasePackage() {
		return basePackage;
	}
	public void setBasePackage(String basePackage) {
		this.basePackage = basePackage;
	}

	static class EnumConverter<T extends IEnum> implements Converter<String, T> {
		
		public T[] ienums;
		
		public EnumConverter(Class<T> clazz) {
			ienums = clazz.getEnumConstants();
		}

		@Override
		public T convert(String source) {
			if(StringUtil.isBlank(source)) {
				return null;
			}
			if(ienums == null) {
				return null;
			}
			int num = Integer.parseInt(source);
			for(T ienum : ienums) {
				if(ienum.getNum() == num) {
					return ienum;
				}
			}
			return null;
		}
	}
}
