package com.sp.blinddate.module.order.controller;

import com.sp.RedisServer;
import com.sp.annotation.Authentication;
import com.sp.blinddate.module.order.entity.Order;
import com.sp.blinddate.module.order.ienum.PayTypeEnum;
import com.sp.blinddate.module.order.param.OrderParam;
import com.sp.blinddate.module.order.service.IOrderService;
import com.sp.blinddate.module.order.vo.OrderVO;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.utils.BeanUtil;
import com.sp.blinddate.utils.EnvironmentUtil;
import com.sp.blinddate.utils.UAParser;
import com.sp.blinddate.utils.WebUtil;
import com.sp.http.MJson;
import com.sp.plugin.pay.wxpay.dto.WXPayAppPayUnifiedDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;
import sun.management.resources.agent;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.net.URLDecoder;

import static wxsdk.WXPayUtil.MD5;


/**
 * @author apple
 */
@Api(tags = {"订单域-订单相关接口"})
@Controller
@RequestMapping("/api/order")
public class OrderController {

    @Autowired
    private RedisServer redisServer;

    @Autowired
    private IOrderService orderService;

    @ApiOperation(value = "2、订单查询-查询订单号查询订单", notes = "订单查询-查询订单号查询订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", required = true, dataType = "string"),
    })
    @RequestMapping(path = "/query/v1", method = RequestMethod.POST)
    @ResponseBody
    public MJson<OrderVO> query(
            @NotNull(message = "订单号为空") String orderNo) {
        OrderVO orderVO = new OrderVO();
        Order orderPO = orderService.queryOrder(orderNo);
        BeanUtil.copyBean(orderPO, orderVO);
        return MJson.ok(orderVO);
    }

    @ApiOperation(value = "1、订单请求支付-统一下单", notes = "订单请求支付-统一下单")
    @RequestMapping(path = "/createOrder/v1", method = RequestMethod.POST)
    @Authentication
    @ResponseBody
    public MJson<?> createOrder(
            @ApiIgnore User user,
            @ApiIgnore HttpServletRequest request) throws Exception {
        String clientIp = WebUtil.getClientIp(request);
        OrderParam orderParam = new OrderParam();
        orderParam.setBusiOrderType("vip");
        orderParam.setChannel("blinddate");
        orderParam.setQuantity(1);
        orderParam.setPayType(PayTypeEnum.WXPayProgram);
        BigDecimal vipPrice = redisServer.get(UserKeyPrefix.userVipPrivaliagePrice, "vip", BigDecimal.class);
        orderParam.setOriginalTotalAmount(vipPrice);
        orderParam.setCurrentTotalAmount(vipPrice);
        Order order = orderService.createOrder(user, orderParam, clientIp);
        WXPayAppPayUnifiedDTO o =(WXPayAppPayUnifiedDTO) orderService.unifiedPay(user.getOpenId(),order, orderParam.getPayType(), clientIp);
        o.setPackageValue("prepay_id="+o.getPrepayid());
        StringBuffer sb = new StringBuffer();
        sb.append("appId=").append(o.getAppid());
        sb.append("&nonceStr=").append(o.getNoncestr());
        sb.append("&package=").append(o.getPackageValue());
        sb.append("&signType=").append("MD5");
        sb.append("&timeStamp=").append(o.getTimestamp());
        sb.append("&key=").append(EnvironmentUtil.getProperty("wxpay.key"));
        String sign = MD5(sb.toString()).toUpperCase();
        o.setPaySign(sign);
        return MJson.ok(o);
    }
}
