package com.sp.blinddate.module.user.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.sp.RedisServer;
import com.sp.annotation.Authentication;
import com.sp.blinddate.base.vo.AddressComponent;
import com.sp.blinddate.base.vo.BaseEnumVO;
import com.sp.blinddate.common.BaseReviewIenum;
import com.sp.blinddate.module.chlidinfo.service.ChildInfoService;
import com.sp.blinddate.module.recommand.vo.RecommandVO;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.entity.UserChildInfo;
import com.sp.blinddate.module.user.ienum.*;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.blinddate.module.user.param.UserFullInfoParam;
import com.sp.blinddate.module.user.service.IUserChildInfoService;
import com.sp.blinddate.module.user.vo.ChildBlindInfoBase;
import com.sp.blinddate.module.user.vo.ChildBlindInfoDetail;
import com.sp.blinddate.utils.EnumUtil;
import com.sp.http.MJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户信息表 前端控制器
 * </p>
 *
 * @author 十一
 * @since 2019-11-13
 */
@RestController
@RequestMapping("/user")
@Validated
@Api(tags = "用户信息域",value = "用户基本信息相关")
public class UserChildInfoController {

    @Autowired
    private IUserChildInfoService userChildInfoService;

    @Autowired
    private RedisServer redisServer;

    @Autowired
    private ChildInfoService childInfoService;

    @ApiOperation(value = "获取可选择录入的用户信息",notes = "获取可选择录入的用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(
                        name = "type",
                        value = "获取具体某一项需要选择的信息：学历：education，关系：relation，婚姻状况，" +
                                "marital，婚房：house，是否有车：car，性别：gender,身高：height,收入：income，年龄：age，" +
                                "职业：carrer",
                        paramType = "query",
                        required = true,
                        dataType = "String")
    })
    @GetMapping("getSelectItemByType")
    public MJson<List<BaseEnumVO>> getSelectItemByType(String type) {
        List<BaseEnumVO> list;
        switch (type) {
            case "education":
                list = EnumUtil.enumToList(EducationEnum.values());
                break;
            case "relation":
                list = EnumUtil.enumToList(RelationEnum.values());
                break;
            case "marital":
                list = EnumUtil.enumToList(MaritalStatusEnum.values());
                break;
            case "house":
                list = EnumUtil.enumToList(HouseEnum.values());
                break;
            case "car":
                list = EnumUtil.enumToList(CarEnum.values());
                break;
            case "height":
                list = EnumUtil.enumToList(UserHegihtEnum.values());
                break;
            case "income":
                list = EnumUtil.enumToList(IncomeEnum.values());
                break;
            case "age":
                list = EnumUtil.enumToList(AgeEnum.values());
                break;
            case "carrer":
                list = EnumUtil.enumToList(CarrerEnum.values());
                break;
            default:
                list = EnumUtil.enumToList(GenderEnum.values());
                break;

        }
        return MJson.ok(list);
    }

    @ApiOperation(value = "上传用户图片", notes = "上传用户图片")
    @RequestMapping(value = "uploadUserPic", method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pic", value = "图片名称", paramType = "query",
                    required = true, dataType = "file")
    })
    public MJson<String> uploadUserPic(@NotNull(message = "上传文件为空") MultipartFile pic) {
        String url = userChildInfoService.uploadUserPic(pic);
        return MJson.ok(url);
    }

    @ApiOperation(value = "保存用户信息",notes = "保存用户信息")
    @RequestMapping(value = "saveInfo",method = RequestMethod.POST)
    @Authentication
    @ResponseBody
    public MJson<Object> saveInfo(@ApiIgnore User user,
                                  @Validated UserFullInfoParam fullInfoParam){
        userChildInfoService.saveInfo(user, fullInfoParam);
        return MJson.ok();
    }
    
    
    @ApiOperation(value = "子女相亲信息", notes = "子女相亲信息")
    @RequestMapping(value = "child/blinddate/base", method = RequestMethod.GET, produces = "application/json")
    @Authentication
    public MJson<Map> getBaseInfo(@ApiIgnore User user){
        List<ChildBlindInfoBase> infoBase = userChildInfoService.getBaseInfo(user);
        Map<String,Object> map = new HashMap<>();
        AddressComponent vo = redisServer.get(UserKeyPrefix.userLocationCity, user.getId()+"", AddressComponent.class);
        map.put("list",infoBase);
        map.put("province", vo.getProvince());
        map.put("city", vo.getCity());
        return MJson.ok(map);
    }

    @ApiOperation(value = "子女的详细信息", notes = "子女的详细信息")
    @RequestMapping(value = "child/blinddate/detail", method = RequestMethod.GET, produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userInfoId", value = "用户信息id", paramType = "query",
                    required = true, dataType = "long")
    })
    public MJson<ChildBlindInfoDetail> getdetailInfo(@Validated @NotNull(message = "用户信息id为空") Long userInfoId){
        ChildBlindInfoDetail infoBase = userChildInfoService.getdetailInfo(userInfoId);
        return MJson.ok(infoBase);
    }


    @ApiOperation(value = "verify", notes = "verify")
    @RequestMapping(value = "verify", method = RequestMethod.GET, produces = "application/json")
    public MJson<Object> verify(){
        QueryWrapper<UserChildInfo> qw = new QueryWrapper<>();
        List<UserChildInfo> list = userChildInfoService.list(qw);
        for (UserChildInfo info : list) {
            childInfoService.verifyUserInfo(info.getId(), BaseReviewIenum.Pass);
        }
        return MJson.ok();
    }

    @ApiOperation(value = "获取当前用户信息的编辑信息", notes = "获取当前用户信息的编辑信息")
    @RequestMapping(value = "getUserEditInfo", method = RequestMethod.GET, produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userInfoId", value = "用户信息id", paramType = "query", required = true, dataType = "long"),
    })
    public MJson<UserFullInfoParam> getUserEditInfo(
           @Validated @NotNull(message = "用户信息id为空") Long userInfoId){
        UserFullInfoParam param = childInfoService.getUserEditInfo(userInfoId);
        return MJson.ok(param);
    }

    @ApiOperation(value = "根据用户id获取用户的子女信息，只包含审核过的用户子女信息",
            notes = "根据用户id获取用户的子女信息，只包含审核过的用户子女信息")
    @RequestMapping(value = "userserChildInfo/userid", method = RequestMethod.GET, produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "query", required = true, dataType = "long"),
    })
    public MJson<List<RecommandVO>> getUserInfosByUserId(@Validated @NotNull(message = "用户id为空") Long userId){
        List<RecommandVO> param = childInfoService.getUserInfosByUserId(userId);
        return MJson.ok(param);
    }
}

