package com.sp.blinddate.module.user.controller;


import com.sp.annotation.Authentication;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.param.EncryUserInfoParam;
import com.sp.blinddate.module.user.service.IUserService;
import com.sp.blinddate.module.user.vo.MineInfoVO;
import com.sp.blinddate.module.user.vo.UserLoginVO;
import com.sp.http.MJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

/**
 * <p>
 * 用户账户表 前端控制器
 * </p>
 *
 * @author 十一
 * @since 2019-11-13
 */
@RestController
@RequestMapping("/user")
@Api(tags = "用户域",value = "用户信息相关")
public class UserController {

    @Autowired
    private IUserService userService;

    @ApiOperation(value = "获取微信的openid", notes = "获取微信的openid")
    @RequestMapping(value = "getUserOpenId", method = RequestMethod.POST, produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "用户登录凭证", paramType = "query", required = true, dataType = "long"),
    })
    public MJson<String>login(String code){
        String openId = userService.loginByCode(code);
        return MJson.ok(openId);
    }

    @ApiOperation(value = "微信登陆", notes = "微信登陆")
    @RequestMapping(value = "login", method = RequestMethod.POST)
    @ResponseBody
    public MJson<UserLoginVO> login(@Validated EncryUserInfoParam param) {
        UserLoginVO vo = userService.login(param);
        return MJson.ok(vo);
    }

    @ApiOperation(value = "获取用户绑定的手机号码", notes = "获取用户绑定的手机号码")
    @RequestMapping(value = "getUserBindPhone", method = RequestMethod.POST)
    @Authentication
    public MJson<String> getUserBindPhone(@ApiIgnore User user,@Validated EncryUserInfoParam param) {
        String phoneNumber = userService.getUserBindPhone(user,param);
        return MJson.ok(phoneNumber);
    }

    @ApiOperation(value = "'我的'信息", notes = "我的信息")
    @RequestMapping(value = "mine", method = RequestMethod.GET, produces = "application/json")
    public MJson<MineInfoVO> getMineInfo(@ApiIgnore User user){
        if(user == null){
            return MJson.ok();
        }
        MineInfoVO vo = userService.getMineInfo(user);
        return MJson.ok(vo);
    }
}
