package com.sp.blinddate.web.exceptionhandler;

import com.aliyun.oss.ServiceException;
import com.sp.blinddate.exception.AuthorityException;
import com.sp.blinddate.result.ResultJson;
import com.sp.exception.*;
import com.sp.http.MJson;
import io.jsonwebtoken.JwtException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.validation.ConstraintViolationException;
import java.sql.SQLIntegrityConstraintViolationException;

/**
 * @Author: 十一
 * @Date: 2019/8/29 16:20
 * @Descrption 异常统一处理
 **/
@ControllerAdvice
public class CentralizedExceptionHandler {

    private static final Logger LOG = LoggerFactory.getLogger(CentralizedExceptionHandler.class);

    /**
     * 登陆类型异常处理
     **/
    @ExceptionHandler(LoginException.class)
    @ResponseBody
    public ResultJson loginException(LoginException ex) {
        ResultJson result = ResultJson.LOGIN_ERROR_RESULT;
        LOG.error(result.getMsg()+"{}",ex.getMessage());
        return ResultJson.LOGIN_ERROR_RESULT;
    }

    /**
     * 登陆类型异常处理
     **/
    @ExceptionHandler(BusinessException.class)
    @ResponseBody
    public MJson loginException(BusinessException ex) {
        LOG.error("业务异常：{}",ex.getMessage());
        return MJson.fail(ex.getMessage());
    }

    /**
     * 功能描述 用户未登陆异常处理
     *
     * @param ex
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46
     */
    @ExceptionHandler(AuthenticateException.class)
    @ResponseBody
    public ResultJson authenticateException(AuthenticateException ex) {
        ResultJson resultJson = ResultJson.AUTH_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs(ex.getMessage());
        LOG.error("用户授权异常：{}",result.getMsg());
        return result;
    }

    /**
     * 功能描述 参数异常
     *
     * @param ex
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46
     */
    @ExceptionHandler(ParamException.class)
    @ResponseBody
    public ResultJson authenticateException(ParamException ex) {
        ResultJson resultJson = ResultJson.PARAM_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs(ex.getMessage());
        LOG.error("参数异常：{}",result.getMsg());
        return result;
    }

    /**
     * 功能描述 请求参数校验异常
     *
     * @param ex
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46
     */
    @ExceptionHandler(ConstraintViolationException.class)
    @ResponseBody
    public ResultJson authenticateException(ConstraintViolationException ex) {
        ResultJson resultJson = ResultJson.PARAM_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs(ex.getMessage());
        LOG.error("请求参数校验异常:{}",result.getMsg());
        return result;
    }

    /**
     * 功能描述 参数校验异常
     *
     * @param ex
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseBody
    public ResultJson authenticateException(MethodArgumentNotValidException ex) {
        ResultJson resultJson = ResultJson.PARAM_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs(ex.getBindingResult().toString());
        LOG.error("请求参数校验异常:{}",result.getMsg());
        return result;
    }

    /**
     *功能描述 数据库操作异常
     * @author 十一
     * @return com.sp.friendship.http.ResultJson
     * @date 2019/9/2 14:46  SQLException
     */
    @ExceptionHandler(SQLIntegrityConstraintViolationException.class)
    @ResponseBody
    public ResultJson constrainvoException(){
        ResultJson resultJson = ResultJson.MYSQL_DUPLICATE_OP_RESUTL;
        //虽然是重复操作，但是对用户来说是成功的
        ResultJson result = resultJson.fillArgs("操作成功");
        LOG.error("数据库操作异常：{}",result.getMsg());
        return result;
    }


    /**
     * 功能描述 参数绑定异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46
     */
    @ExceptionHandler(BindException.class)
    @ResponseBody
    public ResultJson constrainvoException(BindException e) {
        ResultJson resultJson = ResultJson.PARAM_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs(e.getAllErrors().get(0).getDefaultMessage());
        LOG.error("请求参数绑定异常:{}",result.getMsg());
        return result;
    }

    /**
     * 功能描述 微信登陆异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46  SQLException
     */
    @ExceptionHandler(WechatException.class)
    @ResponseBody
    public ResultJson constrainvoException(WechatException e) {
        ResultJson result = new ResultJson();
        result.setCode(MJson.fail().getCode());
        result.setMsg(e.getMessage());
        LOG.error("微信登陆异常:{}",result.getMsg());
        return result;
    }

    /**
     * 功能描述 短信登陆异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46  SQLException
     */
    @ExceptionHandler(ServiceException.class)
    @ResponseBody
    public ResultJson constrainvoException(ServiceException e) {
        ResultJson result = new ResultJson();
        result.setCode(MJson.fail().getCode());
        result.setMsg(e.getMessage());
        LOG.error("短信登陆异常",result.getMsg());
        return result;
    }

    /**
     * 功能描述 用户手机号码绑定异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46  SQLExcen
     */
    @ExceptionHandler(PhoneUnbindException.class)
    @ResponseBody
    public ResultJson phoneUnbindException(PhoneUnbindException e) {
        ResultJson resultJson = ResultJson.USER_PHONE_UNBIND_RESULT;
        ResultJson result = resultJson.fillArgs(e.getMessage());
        LOG.error("用户手机号码绑定异常:{}",result.getMsg());
        return result;

    }

    /**
     * 功能描述 用户未完成基本信息异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46  SQLException
     */
    @ExceptionHandler(UserInfoException.class)
    @ResponseBody
    public ResultJson userInfoImproveException(UserInfoException e) {
        ResultJson resultJson = ResultJson.USER_INFO_IMPROVE_RESULT;
        ResultJson result = resultJson.fillArgs(e.getMessage());
        LOG.error("用户未完成基本信息异常：{}",result.getMsg());
        return result;
    }
    /**
     * 功能描述 校验token异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46  SQLException
     */
    @ExceptionHandler(JwtException.class)
    @ResponseBody
    public ResultJson userTokenException(JwtException e) {
        ResultJson resultJson = ResultJson.AUTH_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs("用户身份认证失败");
        LOG.error("校验token异常：{}",result.getMsg());
        return result;
    }

    /**
     * 功能描述 校验VIP异常
     *
     * @return com.sp.friendship.http.ResultJson
     * @author 十一
     * @date 2019/9/2 14:46  SQLException
     */
    @ExceptionHandler(AuthorityException.class)
    @ResponseBody
    public ResultJson userTokenException(AuthorityException e) {
        ResultJson resultJson = ResultJson.VIP_ERROR_RESULT;
        ResultJson result = resultJson.fillArgs(e.getMessage());
        LOG.error("校验token异常：{}",result.getMsg());
        return result;
    }


//
//    /**
//     *功能描述 处理空指针异常
//     * @author 十一
//     * @date 2019/11/30 14:15
//     */
//    @ExceptionHandler(NullPointerException.class)
//    @ResponseBody
//    public ResultJson userTokenException(NullPointerException e) {
//        ResultJson resultJson = new ResultJson();
//        resultJson.setCode(500);
//        resultJson.setMsg("业务处理异常");
//        LOG.error("空指针错误",e.getMessage());
//        return resultJson;
//    }



}
