package com.sp.blinddate.web.interceptor;

import com.sp.RedisServer;
import com.sp.annotation.Authentication;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.exception.AuthenticateException;
import com.sp.utils.StringUtil;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Descrption
 * @Author: 十一
 * @Date: 2019/11/19 15:26
 **/
public class AuthIntercepter extends HandlerInterceptorAdapter {

    private RedisServer redisServer;

    public AuthIntercepter(RedisServer redisServer) {
        this.redisServer = redisServer;
    }

    public static final String TOKEN = "My-Token";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        //如果用户的请求中带有token，先校验token
        String token = request.getHeader(TOKEN);
        User user = null;
        Boolean loginFlag = false;
        if (!StringUtil.isBlank(token)) {
           user = redisServer.get(UserKeyPrefix.userInfoLoginToken, token, User.class);
           if(user != null){
               loginFlag = true;
               //延长key的过期时间
               redisServer.resetExpireSeconds(UserKeyPrefix.userInfoLoginToken,token,UserKeyPrefix.userInfoLoginToken.getExpireSeconds());
           }
        }
        if (handler instanceof HandlerMethod) {
            HandlerMethod hm = (HandlerMethod) handler;
            Authentication auth = hm.getMethod().getAnnotation(Authentication.class);
            //如果访问的handler需要登陆认证
            if (auth != null) {
                if(!loginFlag){
                    throw new AuthenticateException("登陆异常");
                }
            }
        }
        return super.preHandle(request, response, handler);
    }

}
