package com.sp.blinddate.web.mvc;

import com.sp.RedisServer;
import com.sp.blinddate.module.user.entity.User;
import com.sp.blinddate.module.user.keyprefix.UserKeyPrefix;
import com.sp.exception.AuthenticateException;
import com.sp.utils.StringUtil;
import org.springframework.core.MethodParameter;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

/**
 * @Author: 十一
 * @Date: 2019/9/2 16:35
 * @Descrption
 **/
public class CurrentUserMethodArgumentResolver implements HandlerMethodArgumentResolver {

    private RedisServer redisServer;

    public CurrentUserMethodArgumentResolver(RedisServer redisServer) {
        this.redisServer = redisServer;
    }

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        if(parameter.getParameterType() == User.class){
            return true;
        }
        return false;
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer,
                                  NativeWebRequest webRequest, WebDataBinderFactory binderFactory){
        String token = webRequest.getHeader("My-Token");
        if(StringUtil.isBlank(token)){
            throw new AuthenticateException("登陆异常");
        }
        User user = redisServer.get(UserKeyPrefix.userInfoLoginToken, token, User.class);
        if(user == null){
            throw new AuthenticateException("登陆异常");
        }
        return user;
    }
}